C
C    This program adds the values of the counters in a number of dump
C  files generated from independent runs of PENCYL. It allows the user
C  to run the same problem on several processors, using different seeds
C  of the random number generator, and then combine the results to
C  produce a single set of output files, with accumulated statistics.
C  It works as an effective "poor man's" parallelisation device.
C
C    The program reads the list of relative paths and filenames of the
C  dump files from a text file (standard input, unit 5, one filename in
C  each line). All dump files must correspond to the same problem, that
C  is, they must result from PENCYL simulations with input files that
C  differ only in the seeds of the random number generator. The
C  responsibility of ensuring that the dump files do correspond to the
C  same problem rests with the user.
C
C                                        Francesc Salvat. August, 2014.
C


C  >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
      MODULE TRACK_mod   ! Particle TRACK variables.
C  ----  Energy, position, direction, and weight.
      DOUBLE PRECISION :: E,X,Y,Z,U,V,W,WGHT
C  ----  Particle type, current body (geometry parameter), and material.
      INTEGER*4 :: KPAR,IBODY,MAT
C  ----  Particle history flags.
      INTEGER*4, DIMENSION (5) :: ILB
C  ****  Photon polarisation
C  ----  Polarised photons if IPOL=1, otherwise unpolarised photons.
      INTEGER*4 :: IPOL=0
C  ----  Stokes parameters.
      DOUBLE PRECISION :: SP1,SP2,SP3
C  ****  The particle age (time elapsed since the start of the shower)
C        is recorded when LAGE=.TRUE.
      LOGICAL :: LAGE =.FALSE.
      DOUBLE PRECISION :: PAGE=0.0D0
      END MODULE TRACK_mod
C  <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<


C  *********************************************************************
C                       MAIN PROGRAM
C  *********************************************************************
C
      USE TRACK_mod
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER*4 (I-N)
      CHARACTER TITLE*120,TITLE2*120,PFILER*40,OFILE*20,DATE23*23
      CHARACTER*120 BUF2
      LOGICAL LSPEC,LEMERG,LWOODC
      DIMENSION ISEED1(100),ISEED2(100)
      DIMENSION AUX1(1000),AUX2(500,500)
      PARAMETER (PI=3.1415926535897932D0, TWOPI=2.0D0*PI,
     1  RA2DE=180.0D0/PI)
C
C  ****  Cylindrical geometry.
C
      PARAMETER (NLAM=100,NCYM=50,NBDM=NLAM*NCYM)
      COMMON/CYLGEO/XG(NLAM),YG(NLAM),ZG(NLAM),RG(NLAM,NCYM),
     1  RG2(NLAM,NCYM),RMAX,RMAX2,IBOD(NLAM,NCYM),MATER(NLAM,NCYM),
     1  ILAY(NBDM),ICYL(NBDM),NLAY,NCYL(NLAM),NBOD
      COMMON/CYLAUX/INOUT,KLAY,KCYL
C  ****  Source energy spectrum.
      PARAMETER (NSEM=1000)
      DIMENSION ES(NSEM),PTS(NSEM),SHIST(NSEM)
C
C  ************  Counters.
C
C  ****  Primary particle counters.
      DIMENSION
     1  PRIM(3),PRIM2(3),DPRIM(3),     ! Numbers of IEXIT particles.
     1  SEC(3,3),SEC2(3,3),DSEC(3,3),  ! Generated secondary particles.
     1  AVTL(3),AVTL2(3),     ! Total track length (in material media).
     1  AVW(2),AVW2(2),DAVW(2),        ! Final polar director cosine.
     1  AVA(2),AVA2(2),DAVA(2),        ! Final polar angle.
     1  AVE(2),AVE2(2),DAVE(2),        ! Final energy.
     1  AVI(3,8),AVI2(3,8)             ! Number of interaction events.
C
      DATA PRIM,PRIM2,SEC,SEC2,AVTL,AVTL2,AVW,AVW2,AVA,AVA2,AVE,AVE2,
     1  AVI,AVI2/90*0.0D0/
C
      DIMENSION WSEC(3,3),WSEC2(3,3),WAVTL(3),WAVTL2(3),WAVW(2),
     1  WAVW2(2),WAVA(2),WAVA2(2),WAVE(2),WAVE2(2),WAVI(3,8),WAVI2(3,8)
C
C  ****  Continuous distributions (default output).
C
      DIMENSION BSZ(NLAM),RBSZ(NLAM)
C  ---- Deposited energies in various bodies.
      DIMENSION TDEBO(NBDM),TDEBO2(NBDM)
C  ----  Track length distributions.
      PARAMETER (NBTLM=200)
      DIMENSION PDTL(3,NBTLM),PDTL2(3,NBTLM)
C  ----  Depth-distributions of deposited energy (dose) and charge.
      PARAMETER (NBZM=200)
      DIMENSION DOSZ(NLAM,NBZM),DOSZ2(NLAM,NBZM)
      DIMENSION CHRZ(NLAM,NBZM),CHRZ2(NLAM,NBZM)
C
C  ****  Continuous distributions (selected by the user).
C
C  ----  2D dose and deposited charge distribution (in up to 10 selected
C        bodies).
      PARAMETER (NDOT=10,NBRM=200)
      DIMENSION KBDOD(NDOT),NZDOD(NDOT),NRDOD(NDOT)
      DIMENSION BZDOD(NDOT),BRDOD(NDOT)
      DIMENSION DOSE(NDOT,NBZM,NBRM),DOSE2(NDOT,NBZM,NBRM),
     1          DOSEP(NDOT,NBZM,NBRM)
C
      DIMENSION CHAR(NDOT,NBZM,NBRM),CHAR2(NDOT,NBZM,NBRM),
     1          CHARP(NDOT,NBZM,NBRM)
C  ****  Spatial distributions of emerging particles at the upper and
C        lower planes.
      PARAMETER (NBRT=200,NBXY=2*NBRT)
      DIMENSION PEXY(2,3,NBXY,NBXY),PEXY2(2,3,NBXY,NBXY),
     1  PEXYP(2,3,NBXY,NBXY)
      DIMENSION PER(2,3,NBRT),PER2(2,3,NBRT),PERP(2,3,NBRT)
C
      DATE23='runs at different times'
C
C  ************  Read the first dump file.
C
      IDMP=1
      READ(5,'(A)',ERR=300,END=300) PFILER
      OPEN(9,FILE=PFILER)
      WRITE(6,'(A,A)') '  Reading the dump file ',PFILER
C
      READ (9,*,ERR=300,END=300) SHN,CPUT
      READ (9,'(A65)',ERR=300) TITLE
      READ (9,*,ERR=300) ISEED1(IDMP),ISEED2(IDMP)
      READ (9,*,ERR=300) KPARP,NSEB,LSPEC
      IF(LSPEC) THEN
        READ (9,*,ERR=300) (ES(I),I=1,NSEB+1)
        READ (9,*,ERR=300) (PTS(I),I=1,NSEB+1)
        READ (9,*,ERR=300) (SHIST(I),I=1,NSEB+1)
      ENDIF
      READ (9,*,ERR=300) (PRIM(I),I=1,3)
      READ (9,*,ERR=300) (PRIM2(I),I=1,3)
      READ (9,*,ERR=300) ((SEC(K,I),I=1,3),K=1,3)
      READ (9,*,ERR=300) ((SEC2(K,I),I=1,3),K=1,3)
      READ (9,*,ERR=300) (AVTL(I),I=1,3)
      READ (9,*,ERR=300) (AVTL2(I),I=1,3)
      READ (9,*,ERR=300) (AVW(I),I=1,2)
      READ (9,*,ERR=300) (AVW2(I),I=1,2)
      READ (9,*,ERR=300) (AVA(I),I=1,2)
      READ (9,*,ERR=300) (AVA2(I),I=1,2)
      READ (9,*,ERR=300) (AVE(I),I=1,2)
      READ (9,*,ERR=300) (AVE2(I),I=1,2)
      READ (9,*,ERR=300) ((AVI(I,J),J=1,8),I=1,3)
      READ (9,*,ERR=300) ((AVI2(I,J),J=1,8),I=1,3)
C
      READ (9,*,ERR=300) NLAY
      READ (9,*,ERR=300) (XG(I),I=1,NLAY)
      READ (9,*,ERR=300) (YG(I),I=1,NLAY)
      READ (9,*,ERR=300) (ZG(I),I=1,NLAY+1)
      READ (9,*,ERR=300) (NCYL(I),I=1,NLAY)
      READ (9,*,ERR=300) ((RG(I,J),J=1,NCYL(I)),I=1,NLAY)
C
      READ (9,*,ERR=300) NBZ,(BSZ(I),I=1,NLAY)
      READ (9,*,ERR=300) ((DOSZ(I,J),J=1,NBZ),I=1,NLAY)
      READ (9,*,ERR=300) ((DOSZ2(I,J),J=1,NBZ),I=1,NLAY)
      READ (9,*,ERR=300) ((CHRZ(I,J),J=1,NBZ),I=1,NLAY)
      READ (9,*,ERR=300) ((CHRZ2(I,J),J=1,NBZ),I=1,NLAY)
C
      READ (9,*,ERR=300) LWOODC
      READ (9,*,ERR=300) NBTL,TLMIN,BSTL,RBSTL
      READ (9,*,ERR=300) ((PDTL(I,J),J=1,NBTL),I=1,3)
      READ (9,*,ERR=300) ((PDTL2(I,J),J=1,NBTL),I=1,3)
C
      CALL ENANGR(9)  ! Energy and angular distributions.
C
      READ (9,*,ERR=300) LEMERG
      IF(LEMERG) THEN
        READ (9,*,ERR=300) ZG(1),ZG(NLAY+1)
        READ (9,*,ERR=300) NPEX,NPEY
        READ (9,*,ERR=300) XEL,XEU,YEL,YEU,BSEX,BSEY
        READ (9,*,ERR=300)
     1    ((((PEXY(I,J,K,L),L=1,NPEY),K=1,NPEX),J=1,3),I=1,2)
        READ (9,*,ERR=300)
     1    ((((PEXY2(I,J,K,L),L=1,NPEY),K=1,NPEX),J=1,3),I=1,2)
        READ (9,*,ERR=300) NBRE
        READ (9,*,ERR=300) RADM,BSER
        READ (9,*,ERR=300) (((PER(I,J,K),K=1,NBRE),J=1,3),I=1,2)
        READ (9,*,ERR=300) (((PER2(I,J,K),K=1,NBRE),J=1,3),I=1,2)
      ENDIF
C
      READ (9,*,ERR=300) NBOD
      READ (9,*,ERR=300) (TDEBO(I),I=1,NBOD)
      READ (9,*,ERR=300) (TDEBO2(I),I=1,NBOD)
C
      READ (9,*,ERR=300) NED
      IF(NED.GT.0) THEN
        CALL ENDETR(9)  ! Energy-deposition detectors.
      ENDIF
C
      READ (9,*,ERR=300) NDOD
      IF(NDOD.GT.0) THEN
        READ(9,*,ERR=300) (ILAY(I),I=1,NBOD)
        READ(9,*,ERR=300) (ICYL(I),I=1,NBOD)
        READ(9,*,ERR=300) (KBDOD(I),I=1,NDOD)
        READ(9,*,ERR=300) (BRDOD(I),I=1,NDOD)
        READ(9,*,ERR=300) (BZDOD(I),I=1,NDOD)
        READ(9,*,ERR=300) (NRDOD(I),I=1,NDOD)
        READ(9,*,ERR=300) (NZDOD(I),I=1,NDOD)
        READ(9,*,ERR=300)
     1    (((DOSE(I,J,K),K=1,NRDOD(I)),J=1,NZDOD(I)),I=1,NDOD)
        READ(9,*,ERR=300)
     1    (((DOSE2(I,J,K),K=1,NRDOD(I)),J=1,NZDOD(I)),I=1,NDOD)
        READ(9,*,ERR=300)
     1    (((CHAR(I,J,K),K=1,NRDOD(I)),J=1,NZDOD(I)),I=1,NDOD)
        READ(9,*,ERR=300)
     1    (((CHAR2(I,J,K),K=1,NRDOD(I)),J=1,NZDOD(I)),I=1,NDOD)
      ENDIF
      CLOSE(9)
C
C  ************  Read the second and following dump files.
C
  100 CONTINUE
      READ(5,'(A)',ERR=100,END=200) PFILER
      IDMP=IDMP+1
      OPEN(9,FILE=PFILER)
      WRITE(6,'(A,A)') '  Reading the dump file ',PFILER
C
      READ (9,*,ERR=300,END=300) SHNA,CPUTA
      SHN=SHN+SHNA
      CPUT=CPUT+CPUTA
      READ (9,'(A65)',ERR=300) TITLE2
      IF(TITLE2.NE.TITLE) THEN
        WRITE(6,*)
     1    'The dump file is corrupted (the TITLE does not match).'
        STOP 'The dump file is corrupted (the TITLE does not match).'
      ENDIF
      READ (9,*,ERR=300) ISEED1(IDMP),ISEED2(IDMP)
      READ (9,*,ERR=300) KPARP,NSEB,LSPEC
      IF(LSPEC) THEN
        READ (9,*,ERR=300) (ES(I),I=1,NSEB+1)
        READ (9,*,ERR=300) (PTS(I),I=1,NSEB+1)
        READ (9,*,ERR=300) (AUX1(I),I=1,NSEB+1)
        DO I=1,NSEB
          SHIST(I)=SHIST(I)+AUX1(I)
        ENDDO
      ENDIF
C
      READ (9,*,ERR=300) (DPRIM(I),I=1,3)
      DO I=1,3
        PRIM(I)=PRIM(I)+DPRIM(I)
      ENDDO
      READ (9,*,ERR=300) (DPRIM(I),I=1,3)
      DO I=1,3
        PRIM2(I)=PRIM2(I)+DPRIM(I)
      ENDDO
C
      READ (9,*,ERR=300) ((DSEC(K,I),I=1,3),K=1,3)
      DO K=1,3
        DO I=1,3
          SEC(K,I)=SEC(K,I)+DSEC(K,I)
        ENDDO
      ENDDO
      READ (9,*,ERR=300) ((DSEC(K,I),I=1,3),K=1,3)
      DO K=1,3
        DO I=1,3
          SEC2(K,I)=SEC2(K,I)+DSEC(K,I)
        ENDDO
      ENDDO
C
      READ (9,*,ERR=300) (AUX1(I),I=1,3)
      DO I=1,3
        AVTL(I)=AVTL(I)+AUX1(I)
      ENDDO
      READ (9,*,ERR=300) (AUX1(I),I=1,3)
      DO I=1,3
        AVTL2(I)=AVTL2(I)+AUX1(I)
      ENDDO
C
      READ (9,*,ERR=300) (DAVW(I),I=1,2)
      DO I=1,2
        AVW(I)=AVW(I)+DAVW(I)
      ENDDO
      READ (9,*,ERR=300) (DAVW(I),I=1,2)
      DO I=1,2
        AVW2(I)=AVW2(I)+DAVW(I)
      ENDDO
C
      READ (9,*,ERR=300) (DAVA(I),I=1,2)
      DO I=1,2
        AVA(I)=AVA(I)+DAVA(I)
      ENDDO
      READ (9,*,ERR=300) (DAVA(I),I=1,2)
      DO I=1,2
        AVA2(I)=AVA2(I)+DAVA(I)
      ENDDO
C
      READ (9,*,ERR=300) (DAVE(I),I=1,2)
      DO I=1,2
        AVE(I)=AVE(I)+DAVE(I)
      ENDDO
      READ (9,*,ERR=300) (DAVE(I),I=1,2)
      DO I=1,2
        AVE2(I)=AVE2(I)+DAVE(I)
      ENDDO
C
      READ (9,*,ERR=300) ((AUX2(I,J),J=1,8),I=1,3)
      DO J=1,8
        DO I=1,3
         AVI(I,J)=AVI(I,J)+AUX2(I,J)
        ENDDO
      ENDDO
      READ (9,*,ERR=300) ((AUX2(I,J),J=1,8),I=1,3)
      DO J=1,8
        DO I=1,3
         AVI2(I,J)=AVI2(I,J)+AUX2(I,J)
        ENDDO
      ENDDO
C
      READ (9,*,ERR=300) NLAY
      READ (9,*,ERR=300) (XG(I),I=1,NLAY)
      READ (9,*,ERR=300) (YG(I),I=1,NLAY)
      READ (9,*,ERR=300) (ZG(I),I=1,NLAY+1)
      READ (9,*,ERR=300) (NCYL(I),I=1,NLAY)
      READ (9,*,ERR=300) ((RG(I,J),J=1,NCYL(I)),I=1,NLAY)
C
      READ (9,*,ERR=300) NBZ,(BSZ(I),I=1,NLAY)
      READ (9,*,ERR=300) ((AUX2(I,J),J=1,NBZ),I=1,NLAY)
      DO I=1,NLAY
        DO J=1,NBZ
          DOSZ(I,J)=DOSZ(I,J)+AUX2(I,J)
        ENDDO
      ENDDO
      READ (9,*,ERR=300) ((AUX2(I,J),J=1,NBZ),I=1,NLAY)
      DO I=1,NLAY
        DO J=1,NBZ
          DOSZ2(I,J)=DOSZ2(I,J)+AUX2(I,J)
        ENDDO
      ENDDO
C
      READ (9,*,ERR=300) ((AUX2(I,J),J=1,NBZ),I=1,NLAY)
      DO I=1,NLAY
        DO J=1,NBZ
          CHRZ(I,J)=CHRZ(I,J)+AUX2(I,J)
        ENDDO
      ENDDO
      READ (9,*,ERR=300) ((AUX2(I,J),J=1,NBZ),I=1,NLAY)
      DO I=1,NLAY
        DO J=1,NBZ
          CHRZ2(I,J)=CHRZ2(I,J)+AUX2(I,J)
        ENDDO
      ENDDO
C
      READ (9,*,ERR=300) LWOODC
      READ (9,*,ERR=300) NBTL,TLMIN,BSTL,RBSTL
      READ (9,*,ERR=300) ((AUX2(I,J),J=1,NBTL),I=1,3)
      DO I=1,3
        DO J=1,NBTL
          PDTL(I,J)=PDTL(I,J)+AUX2(I,J)
        ENDDO
      ENDDO
      READ (9,*,ERR=300) ((AUX2(I,J),J=1,NBTL),I=1,3)
      DO I=1,3
        DO J=1,NBTL
          PDTL2(I,J)=PDTL2(I,J)+AUX2(I,J)
        ENDDO
      ENDDO
C
      CALL ENANGA(9)  ! Energy and angular distributions.
C
      READ (9,*,ERR=300) LEMERG
      IF(LEMERG) THEN
        READ (9,*,ERR=300) ZG(1),ZG(NLAY+1)
        READ (9,*,ERR=300) NPEX,NPEY
        READ (9,*,ERR=300) XEL,XEU,YEL,YEU,BSEX,BSEY
        READ (9,*,ERR=300)
     1    ((((PEXYP(I,J,K,L),L=1,NPEY),K=1,NPEX),J=1,3),I=1,2)
        DO I=1,2
          DO J=1,3
            DO K=1,NPEX
              DO L=1,NPEY
                PEXY(I,J,K,L)=PEXY(I,J,K,L)+PEXYP(I,J,K,L)
              ENDDO
            ENDDO
          ENDDO
        ENDDO
        READ (9,*,ERR=300)
     1    ((((PEXYP(I,J,K,L),L=1,NPEY),K=1,NPEX),J=1,3),I=1,2)
        DO I=1,2
          DO J=1,3
            DO K=1,NPEX
              DO L=1,NPEY
                PEXY2(I,J,K,L)=PEXY2(I,J,K,L)+PEXYP(I,J,K,L)
              ENDDO
            ENDDO
          ENDDO
        ENDDO
C
        READ (9,*,ERR=300) NBRE
        READ (9,*,ERR=300) RADM,BSER
        READ (9,*,ERR=300) (((PERP(I,J,K),K=1,NBRE),J=1,3),I=1,2)
        DO I=1,2
          DO J=1,3
            DO K=1,NBRE
              PER(I,J,K)=PER(I,J,K)+PERP(I,J,K)
            ENDDO
          ENDDO
        ENDDO
        READ (9,*,ERR=300) (((PERP(I,J,K),K=1,NBRE),J=1,3),I=1,2)
        DO I=1,2
          DO J=1,3
            DO K=1,NBRE
              PER2(I,J,K)=PER2(I,J,K)+PERP(I,J,K)
            ENDDO
          ENDDO
        ENDDO
      ENDIF
C
      READ (9,*,ERR=300) NBOD
      READ (9,*,ERR=300) (AUX1(I),I=1,NBOD)
      DO I=1,NBOD
        TDEBO(I)=TDEBO(I)+AUX1(I)
      ENDDO
      READ (9,*,ERR=300) (AUX1(I),I=1,NBOD)
      DO I=1,NBOD
        TDEBO2(I)=TDEBO2(I)+AUX1(I)
      ENDDO
C
      READ (9,*,ERR=300) NED
      IF(NED.GT.0) THEN
        CALL ENDETA(9)  ! Energy-deposition detectors.
      ENDIF
C
      READ (9,*,ERR=300) NDOD
      IF(NDOD.GT.0) THEN
        READ(9,*,ERR=300) (ILAY(I),I=1,NBOD)
        READ(9,*,ERR=300) (ICYL(I),I=1,NBOD)
        READ(9,*,ERR=300) (KBDOD(I),I=1,NDOD)
        READ(9,*,ERR=300) (BRDOD(I),I=1,NDOD)
        READ(9,*,ERR=300) (BZDOD(I),I=1,NDOD)
        READ(9,*,ERR=300) (NRDOD(I),I=1,NDOD)
        READ(9,*,ERR=300) (NZDOD(I),I=1,NDOD)
        READ(9,*,ERR=300)
     1    (((DOSEP(I,J,K),K=1,NRDOD(I)),J=1,NZDOD(I)),I=1,NDOD)
        DO I=1,NDOD
          DO J=1,NZDOD(I)
            DO K=1,NRDOD(I)
              DOSE(I,J,K)=DOSE(I,J,K)+DOSEP(I,J,K)
            ENDDO
          ENDDO
        ENDDO
        READ(9,*,ERR=300)
     1    (((DOSEP(I,J,K),K=1,NRDOD(I)),J=1,NZDOD(I)),I=1,NDOD)
        DO I=1,NDOD
          DO J=1,NZDOD(I)
            DO K=1,NRDOD(I)
              DOSE2(I,J,K)=DOSE2(I,J,K)+DOSEP(I,J,K)
            ENDDO
          ENDDO
        ENDDO
C
        READ(9,*,ERR=300)
     1    (((CHARP(I,J,K),K=1,NRDOD(I)),J=1,NZDOD(I)),I=1,NDOD)
        DO I=1,NDOD
          DO J=1,NZDOD(I)
            DO K=1,NRDOD(I)
              CHAR(I,J,K)=CHAR(I,J,K)+CHARP(I,J,K)
            ENDDO
          ENDDO
        ENDDO
        READ(9,*,ERR=300)
     1    (((CHARP(I,J,K),K=1,NRDOD(I)),J=1,NZDOD(I)),I=1,NDOD)
        DO I=1,NDOD
          DO J=1,NZDOD(I)
            DO K=1,NRDOD(I)
              CHAR2(I,J,K)=CHAR2(I,J,K)+CHARP(I,J,K)
            ENDDO
          ENDDO
        ENDDO
      ENDIF
      CLOSE(9)
C
      GO TO 100
  200 CONTINUE
C
C  ************  Write counters in a global dump file.
C
      DO KL=1,NLAY
        RBSZ(KL)=1.0D0/BSZ(KL)
      ENDDO
      WGHT0=1.0D0
      TSIM=CPUT
C
      OPEN(9,FILE='all-dump.dat')
      WRITE(9,*) SHN,TSIM
      WRITE(9,'(A65)') TITLE
      WRITE(9,*) ISEED1(IDMP),ISEED2(IDMP)
      WRITE(9,*) KPARP,NSEB,LSPEC
      IF(LSPEC) THEN
        WRITE(9,*) (ES(I),I=1,NSEB+1)
        WRITE(9,*) (PTS(I),I=1,NSEB+1)
        WRITE(9,*) (SHIST(I),I=1,NSEB+1)
      ENDIF
      WRITE(9,*) (PRIM(I),I=1,3)
      WRITE(9,*) (PRIM2(I),I=1,3)
      WRITE(9,*) ((SEC(K,I),I=1,3),K=1,3)
      WRITE(9,*) ((SEC2(K,I),I=1,3),K=1,3)
      WRITE(9,*) (AVTL(I),I=1,3)
      WRITE(9,*) (AVTL2(I),I=1,3)
      WRITE(9,*) (AVW(I),I=1,2)
      WRITE(9,*) (AVW2(I),I=1,2)
      WRITE(9,*) (AVA(I),I=1,2)
      WRITE(9,*) (AVA2(I),I=1,2)
      WRITE(9,*) (AVE(I),I=1,2)
      WRITE(9,*) (AVE2(I),I=1,2)
      WRITE(9,*) ((AVI(I,J),J=1,8),I=1,3)
      WRITE(9,*) ((AVI2(I,J),J=1,8),I=1,3)
C
      WRITE(9,*) NLAY
      WRITE(9,*) (XG(I),I=1,NLAY)
      WRITE(9,*) (YG(I),I=1,NLAY)
      WRITE(9,*) (ZG(I),I=1,NLAY+1)
      WRITE(9,*) (NCYL(I),I=1,NLAY)
      WRITE(9,*) ((RG(I,J),J=1,NCYL(I)),I=1,NLAY)
C
      WRITE(9,*) NBZ,(BSZ(I),I=1,NLAY)
      WRITE(9,*) ((DOSZ(I,J),J=1,NBZ),I=1,NLAY)
      WRITE(9,*) ((DOSZ2(I,J),J=1,NBZ),I=1,NLAY)
      WRITE(9,*) ((CHRZ(I,J),J=1,NBZ),I=1,NLAY)
      WRITE(9,*) ((CHRZ2(I,J),J=1,NBZ),I=1,NLAY)
C
      WRITE(9,*) LWOODC
      WRITE(9,*) NBTL,TLMIN,BSTL,RBSTL
      WRITE(9,*) ((PDTL(I,J),J=1,NBTL),I=1,3)
      WRITE(9,*) ((PDTL2(I,J),J=1,NBTL),I=1,3)
C
      CALL ENANGD(9)  ! Energy and angular distributions.
C
      WRITE(9,*) LEMERG
      IF(LEMERG) THEN
        WRITE(9,*) ZG(1),ZG(NLAY+1)
        WRITE(9,*) NPEX,NPEY
        WRITE(9,*) XEL,XEU,YEL,YEU,BSEX,BSEY
        WRITE(9,*)
     1    ((((PEXY(I,J,K,L),L=1,NPEY),K=1,NPEX),J=1,3),I=1,2)
        WRITE(9,*)
     1    ((((PEXY2(I,J,K,L),L=1,NPEY),K=1,NPEX),J=1,3),I=1,2)
        WRITE(9,*) NBRE
        WRITE(9,*) RADM,BSER
        WRITE(9,*) (((PER(I,J,K),K=1,NBRE),J=1,3),I=1,2)
        WRITE(9,*) (((PER2(I,J,K),K=1,NBRE),J=1,3),I=1,2)
      ENDIF
C
      WRITE(9,*) NBOD
      WRITE(9,*) (TDEBO(I),I=1,NBOD)
      WRITE(9,*) (TDEBO2(I),I=1,NBOD)
C
      WRITE(9,*) NED
      IF(NED.GT.0) THEN
        CALL ENDETD(9)  ! Energy-deposition detectors.
      ENDIF
C
      WRITE(9,*) NDOD
      IF(NDOD.GT.0) THEN
        WRITE(9,*) (ILAY(I),I=1,NBOD)
        WRITE(9,*) (ICYL(I),I=1,NBOD)
        WRITE(9,*) (KBDOD(I),I=1,NDOD)
        WRITE(9,*) (BRDOD(I),I=1,NDOD)
        WRITE(9,*) (BZDOD(I),I=1,NDOD)
        WRITE(9,*) (NRDOD(I),I=1,NDOD)
        WRITE(9,*) (NZDOD(I),I=1,NDOD)
        WRITE(9,*)
     1    (((DOSE(I,J,K),K=1,NRDOD(I)),J=1,NZDOD(I)),I=1,NDOD)
        WRITE(9,*)
     1    (((DOSE2(I,J,K),K=1,NRDOD(I)),J=1,NZDOD(I)),I=1,NDOD)
        WRITE(9,*)
     1    (((CHAR(I,J,K),K=1,NRDOD(I)),J=1,NZDOD(I)),I=1,NDOD)
        WRITE(9,*)
     1    (((CHAR2(I,J,K),K=1,NRDOD(I)),J=1,NZDOD(I)),I=1,NDOD)
      ENDIF
C
      WRITE(9,'(/3X,''*** END ***'')')
      CLOSE(9)
C
C  ------------------------  Print simulation results.
C
C     IEXIT: 1=upbound, 2=downbound, 3=absorbed.
C
      OPEN(27,FILE='pencyl-res.dat')
      TOTN=SHN*WGHT0
      WRITE(27,3000)
 3000 FORMAT(//3X,34('*')/3X,'**   Program PENCYL. Results.   **',
     1  /3X,34('*'))
C
      WRITE(27,1101) DATE23
 1101 FORMAT(/3X,'Date and time: ',A23)
      WRITE(27,'(/3X,A65)') TITLE
C
      WRITE(27,3001) TSIM
 3001 FORMAT(/3X,'Simulation time ......................... ',
     1  1P,E13.6,' sec')
      TAVS=TOTN/TSIM
      WRITE(27,3002) TAVS
 3002 FORMAT(3X,'Simulation speed ........................ ',
     1  1P,E13.6,' showers/sec')
      WRITE(27,3003) TOTN
 3003 FORMAT(//3X,'Simulated primary particles ............. ',
     1  1P,E13.6)
C
      IF(KPARP.EQ.1) WRITE(27,1110)
 1110 FORMAT(/3X,'Primary particles: electrons')
      IF(KPARP.EQ.2) WRITE(27,1111)
 1111 FORMAT(/3X,'Primary particles: photons')
      IF(KPARP.EQ.3) WRITE(27,1112)
 1112 FORMAT(/3X,'Primary particles: positrons')
C
      WRITE(27,3004) PRIM(1)
 3004 FORMAT(/3X,'Upbound primary particles ............... ',
     1  1P,E13.6)
      WRITE(27,3005) PRIM(2)
 3005 FORMAT(3X,'Downbound primary particles ............. ',
     1  1P,E13.6)
      IF(KPARP.EQ.2) PRIM(3)=TOTN-PRIM(1)-PRIM(2)
      WRITE(27,3006) PRIM(3)
 3006 FORMAT(3X,'Absorbed primary particles .............. ',
     1  1P,E13.6)
C
      FNT=1.0D0/TOTN
      FT=(PRIM(1)+SEC(KPARP,1))*FNT
      ERR1=3.0D0*FNT*SQRT(ABS(PRIM2(1)-PRIM(1)**2*FNT))
      ERR2=3.0D0*FNT*SQRT(ABS(SEC2(KPARP,1)-SEC(KPARP,1)**2*FNT))
      ERR=ERR1+ERR2
      WRITE(27,3007) FT,ERR
 3007 FORMAT(/3X,'Upbound fraction ................... ',
     1  1P,E13.6,' +-',E8.1)
      FB=(PRIM(2)+SEC(KPARP,2))*FNT
      ERR1=3.0D0*FNT*SQRT(ABS(PRIM2(2)-PRIM(2)**2*FNT))
      ERR2=3.0D0*FNT*SQRT(ABS(SEC2(KPARP,2)-SEC(KPARP,2)**2*FNT))
      ERR=ERR1+ERR2
      WRITE(27,3008) FB,ERR
 3008 FORMAT(3X,'Downbound fraction ................. ',
     1  1P,E13.6,' +-',E8.1)
      FA=PRIM(3)*FNT
      ERR=3.0D0*FNT*SQRT(ABS(PRIM2(3)-PRIM(3)**2*FNT))
      WRITE(27,3009) FA,ERR
 3009 FORMAT(3X,'Absorption fraction ................ ',
     1  1P,E13.6,' +-',E8.1)
C
      DO K=1,3
        DO I=1,3
          WSEC2(K,I)=3.0D0*FNT*SQRT(ABS(SEC2(K,I)-SEC(K,I)**2*FNT))
          WSEC(K,I)=SEC(K,I)*FNT
        ENDDO
      ENDDO
      WRITE(27,3010)
     1  WSEC(1,1),WSEC(2,1),WSEC(3,1),WSEC2(1,1),WSEC2(2,1),WSEC2(3,1),
     1  WSEC(1,2),WSEC(2,2),WSEC(3,2),WSEC2(1,2),WSEC2(2,2),WSEC2(3,2),
     1  WSEC(1,3),WSEC(2,3),WSEC(3,3),WSEC2(1,3),WSEC2(2,3),WSEC2(3,3)
 3010 FORMAT(/3X,'Secondary-particle generation probabilities:',
     1  /19X,46('-'),
     1  /19X,'|  electrons   |   photons    |  positrons   |',1P,
     1  /3X,62('-')/3X,'|   upbound     |',3(E13.6,1X,'|'),
     1  /3X,'|               |',3('  +-',E8.1,2X,'|'),
     1  /3X,62('-')/3X,'|   downbound   |',3(E13.6,1X,'|'),
     1  /3X,'|               |',3('  +-',E8.1,2X,'|'),
     1  /3X,62('-')/3X,'|   absorbed    |',3(E13.6,1X,'|'),
     1  /3X,'|               |',3('  +-',E8.1,2X,'|'),
     1  /3X,62('-'))
C
C  ************  Average values for primary particles.
C
      DO I=1,3
        DF=1.0D0/MAX(PRIM(I),1.0D0)
        WAVTL2(I)=3.0D0*DF*SQRT(ABS(AVTL2(I)-AVTL(I)**2*DF))
        WAVTL(I)=AVTL(I)*DF
        IF(I.LT.3) THEN
          WAVE2(I)=3.0D0*DF*SQRT(ABS(AVE2(I)-AVE(I)**2*DF))
          WAVE(I)=AVE(I)*DF
          WAVW2(I)=3.0D0*DF*SQRT(ABS(AVW2(I)-AVW(I)**2*DF))
          WAVW(I)=AVW(I)*DF
          WAVA2(I)=3.0D0*DF*RA2DE*SQRT(ABS(AVA2(I)-AVA(I)**2*DF))
          WAVA(I)=AVA(I)*RA2DE*DF
        ENDIF
        DO KC=1,8
          IF(AVI(I,KC).GT.1.0D-16) THEN
            WAVI2(I,KC)=3.0D0*DF*SQRT(ABS(AVI2(I,KC)-AVI(I,KC)**2*DF))
            WAVI(I,KC)=AVI(I,KC)*DF
          ELSE
            WAVI2(I,KC)=0.0D0
            WAVI(I,KC)=0.0D0
          ENDIF
        ENDDO
      ENDDO
C
      WRITE(27,3011)
 3011 FORMAT(//3X,'Mean number of events per primary track:')
      WRITE(27,3012)
 3012 FORMAT(/6X,'** Upbound primary particles:')
      IF(KPARP.EQ.2) THEN
        WRITE(27,3013) WAVI(1,1),WAVI2(1,1)
 3013   FORMAT(6X,'Coherent (Rayleigh) ............. ',1P,E13.6,
     1    ' +-',E8.1)
        WRITE(27,3014) WAVI(1,2),WAVI2(1,2)
 3014   FORMAT(6X,'Incoherent (Compton) ............ ',1P,E13.6,
     1    ' +-',E8.1)
        WRITE(27,3015) WAVI(1,3),WAVI2(1,3)
 3015   FORMAT(6X,'Photoelectric absorption ........ ',1P,E13.6,
     1    ' +-',E8.1)
        WRITE(27,3016) WAVI(1,4),WAVI2(1,4)
 3016   FORMAT(6X,'Pair production ................. ',1P,E13.6,
     1    ' +-',E8.1)
        WRITE(27,3017) WAVI(1,7),WAVI2(1,7)
 3017   FORMAT(6X,'Delta interactions .............. ',1P,E13.6,
     1    ' +-',E8.1)
      ELSE
        WRITE(27,3018) WAVI(1,1),WAVI2(1,1)
 3018   FORMAT(6X,'Hinges (soft events) ............ ',1P,E13.6,
     1    ' +-',E8.1)
        WRITE(27,3019) WAVI(1,2),WAVI2(1,2)
 3019   FORMAT(6X,'Hard elastic collisions ......... ',1P,E13.6,
     1    ' +-',E8.1)
        WRITE(27,3020) WAVI(1,3),WAVI2(1,3)
 3020   FORMAT(6X,'Hard inelastic collisions ....... ',1P,E13.6,
     1    ' +-',E8.1)
        WRITE(27,3021) WAVI(1,4),WAVI2(1,4)
 3021   FORMAT(6X,'Hard bremsstrahlung emissions ... ',1P,E13.6,
     1    ' +-',E8.1)
        WRITE(27,3022) WAVI(1,5),WAVI2(1,5)
 3022   FORMAT(6X,'Inner-shell ionizations ......... ',1P,E13.6,
     1    ' +-',E8.1)
        WRITE(27,3023) WAVI(1,7),WAVI2(1,7)
 3023   FORMAT(6X,'Delta interactions .............. ',1P,E13.6,
     1    ' +-',E8.1)
      ENDIF
C
      WRITE(27,3024)
 3024 FORMAT(/6X,'** Downbound primary particles:')
      IF(KPARP.EQ.2) THEN
        WRITE(27,3013) WAVI(2,1),WAVI2(2,1)
        WRITE(27,3014) WAVI(2,2),WAVI2(2,2)
        WRITE(27,3015) WAVI(2,3),WAVI2(2,3)
        WRITE(27,3016) WAVI(2,4),WAVI2(2,4)
        WRITE(27,3017) WAVI(2,7),WAVI2(2,7)
      ELSE
        WRITE(27,3018) WAVI(2,1),WAVI2(2,1)
        WRITE(27,3019) WAVI(2,2),WAVI2(2,2)
        WRITE(27,3020) WAVI(2,3),WAVI2(2,3)
        WRITE(27,3021) WAVI(2,4),WAVI2(2,4)
        WRITE(27,3022) WAVI(2,5),WAVI2(2,5)
        WRITE(27,3023) WAVI(2,7),WAVI2(2,7)
      ENDIF
C
      WRITE(27,3025)
 3025 FORMAT(/6X,'** Absorbed primary particles:')
      IF(KPARP.EQ.2) THEN
        WRITE(27,3013) WAVI(3,1),WAVI2(3,1)
        WRITE(27,3014) WAVI(3,2),WAVI2(3,2)
        WRITE(27,3015) WAVI(3,3),WAVI2(3,3)
        WRITE(27,3016) WAVI(3,4),WAVI2(3,4)
        WRITE(27,3017) WAVI(3,7),WAVI2(3,7)
      ELSE
        WRITE(27,3018) WAVI(3,1),WAVI2(3,1)
        WRITE(27,3019) WAVI(3,2),WAVI2(3,2)
        WRITE(27,3020) WAVI(3,3),WAVI2(3,3)
        WRITE(27,3021) WAVI(3,4),WAVI2(3,4)
        WRITE(27,3022) WAVI(3,5),WAVI2(3,5)
        WRITE(27,3023) WAVI(3,7),WAVI2(3,7)
      ENDIF
C
      WRITE(27,3026)
 3026 FORMAT(/3X,'Average final energy:')
      WRITE(27,3027) WAVE(1),WAVE2(1)
 3027 FORMAT(6X,'Upbound primary particles ....... ',1P,E13.6,
     1  ' +-',E8.1,' eV')
      WRITE(27,3028) WAVE(2),WAVE2(2)
 3028 FORMAT(6X,'Downbound primary particles ..... ',1P,E13.6,
     1  ' +-',E8.1,' eV')
C
      IF(.NOT.LWOODC) THEN
        WRITE(27,3029)
 3029   FORMAT(/3X,'Average track length:')
        WRITE(27,3030) WAVTL(1),WAVTL2(1)
 3030   FORMAT(6X,'Upbound primary particles ....... ',1P,E13.6,
     1    ' +-',E8.1,' cm')
        WRITE(27,3031) WAVTL(2),WAVTL2(2)
 3031   FORMAT(6X,'Downbound primary particles ..... ',1P,E13.6,
     1    ' +-',E8.1,' cm')
        WRITE(27,3032) WAVTL(3),WAVTL2(3)
 3032   FORMAT(6X,'Absorbed primary particles ...... ',1P,E13.6,
     1    ' +-',E8.1,' cm')
      ENDIF
C
      WRITE(27,3033)
 3033 FORMAT(/3X,'Mean value of the polar cosine of the exit',
     1  ' direction:')
      WRITE(27,3034) WAVW(1),WAVW2(1)
 3034 FORMAT(6X,'Upbound primary particles ....... ',1P,E13.6,
     1  ' +-',E8.1)
      WRITE(27,3035) WAVW(2),WAVW2(2)
 3035 FORMAT(6X,'Downbound primary particles ..... ',1P,E13.6,
     1  ' +-',E8.1)
C
      WRITE(27,3036)
 3036 FORMAT(/3X,'Mean value of the polar angle of the exit dir',
     1  'ection:')
      WRITE(27,3037) WAVA(1),WAVA2(1)
 3037 FORMAT(6X,'Upbound primary particles ....... ',1P,E13.6,
     1  ' +-',E8.1,' deg')
      WRITE(27,3038) WAVA(2),WAVA2(2)
 3038 FORMAT(6X,'Downbound primary particles ..... ',1P,E13.6,
     1  ' +-',E8.1,' deg')
C
C  ****  Deposited energies.
C
      DF=1.0D0/TOTN
      WRITE(27,3039)
 3039 FORMAT(/3X,'Average deposited energies (per primary shower):',
     1  /6X,'Body',3X,'KL',2X,'KC',4X,'Deposited energy (eV)',3X,
     1  'Efficiency')
      DO KB=1,NBOD
        QER=3.0D0*DF*SQRT(ABS(TDEBO2(KB)-TDEBO(KB)**2*DF))
        QAV=TDEBO(KB)*DF
        IF(QER.GT.1.0D-10*ABS(QAV)) THEN
          EFFIC=QAV**2/((QER/3.0D0)**2*TSIM)
        ELSE
          EFFIC=0.0D0
        ENDIF
        WRITE(27,3040) KB,ILAY(KB),ICYL(KB),QAV,QER,EFFIC
      ENDDO
 3040 FORMAT(6X,I4,3X,I2,2X,I2,2X,1P,E13.6,' +-',E8.1,2X,E9.2)
C
C  ------------------------  Print tallied distributions.
C
      IF(LSPEC) THEN
C
C  ************  Energy spectrum of the source.
C
        OPEN(9,FILE='psource.dat')
        WRITE(9,9000)
 9000 FORMAT(
     1  1X,'#  Results from PENCYL. ',
     1 /1X,'#  Source energy spectrum.',
     1 /1X,'#  1st column: E (eV). 2nd column: spectrum (1/eV).',
     1 /1X,'#  3rd and 4th columns: simul. pdf limits (3SD, 1/eV).')
        PTOT=0.0D0
        WRITE(9,'(1P,4E14.6)') ES(1),PTOT,PTOT,PTOT
        DO KEn=1,NSEB
          PTOT=PTOT+PTS(KEn)
        ENDDO
        DO KEn=1,NSEB
          YAV=SHIST(KEn)*DF
          YERR=3.0D0*SQRT(ABS(YAV*(1.0D0-YAV)*DF))
          EINTL=ES(KEn+1)-ES(KEn)
          IF(EINTL.GT.1.0D-15) THEN
            FACT=1.0D0/EINTL
          ELSE
            FACT=1.0D15
          ENDIF
          WRITE(9,'(1P,4E14.6)') ES(KEn),PTS(KEn)*FACT/PTOT,
     1      (YAV-YERR)*FACT,(YAV+YERR)*FACT
          WRITE(9,'(1P,4E14.6)') ES(KEn+1),PTS(KEn)*FACT/PTOT,
     1      (YAV-YERR)*FACT,(YAV+YERR)*FACT
        ENDDO
        PTOT=0.0D0
        WRITE(9,'(1P,4E14.6)') ES(NSEB+1),PTOT,PTOT,PTOT
        CLOSE(9)
      ENDIF
C
C  ************  Depth-dose.
C
      OPEN(9,FILE='depth-dose.dat')
      WRITE(9,9010)
 9010 FORMAT(
     1     1X,'#  Results from PENCYL. Depth-dose distribution.',
     1    /1X,'#  (integrated over X and Y within the volume of the ',
     1      'material system).',
     1    /1X,'#  1st column: z coordinate (cm).',
     1    /1X,'#  2nd column: depth-dose (eV/(g/cm**2)).',
     1    /1X,'#  3rd column: statistical uncertainty (3 sigma).')
        WRITE(9,9411)
 9411   FORMAT(1X,'#',
     1    /1X,'#  NOTE: The calculated dose distribution is correct',
     1       ' only when the ',
     1    /1X,'#',8X,'Z bins have uniform mass density.',/1X,'#')
      DO KL=1,NLAY
        DO K=1,NBZ
          XX=ZG(KL)+(K-0.5D0)*BSZ(KL)
          YERR=3.0D0*SQRT(ABS(DOSZ2(KL,K)-DOSZ(KL,K)**2*DF))
          YAV=DOSZ(KL,K)*DF*RBSZ(KL)
          YERR=YERR*DF*RBSZ(KL)
          WRITE(9,'(1P,E14.6,E14.6,E10.2,I5)')
     1      XX,MAX(YAV,1.0D-35),MAX(YERR,1.0D-35),KL
        ENDDO
      ENDDO
      CLOSE(9)
C
C  ************  Depth distribution of deposited charge.
C
      OPEN(9,FILE='depth-charge.dat')
      WRITE(9,9020)
 9020 FORMAT(
     1  1X,'#  Results from PENCYL. ',
     1 /1X,'#  Depth distribution of deposited charge.',
     1 /1X,'#  1st column: Z (cm). 4th column: corresponding layer.',
     1 /1X,'#  2nd and 3rd columns:',
     1  ' charge density and STU (e/(cm*primary_particle)).')
      DO KL=1,NLAY
        DO K=1,NBZ
          XX=ZG(KL)+(K-0.5D0)*BSZ(KL)
          YERR=3.0D0*SQRT(ABS(CHRZ2(KL,K)-CHRZ(KL,K)**2*DF))
          YAV=CHRZ(KL,K)*DF*RBSZ(KL)
          YERR=YERR*DF*RBSZ(KL)
          WRITE(9,'(1P,E14.6,E14.6,E10.2,I5)') XX,YAV,YERR,KL
        ENDDO
      ENDDO
      CLOSE(9)
C
C  ************  Track length distributions of primary particles.
C
      IF(.NOT.LWOODC) THEN
      OPEN(9,FILE='track-length.dat')
      WRITE(9,9030)
 9030 FORMAT(
     1  1X,'#  Results from PENCYL. ',
     1 /1X,'#  Track length distribution of primary particles.',
     1 /1X,'#  1st column: track length (cm).',
     1 /1X,'#  2nd and 3rd columns: PDF and STU for upbound particles.',
     1 /1X,'#  4th and 5th columns: PDF and STU for ',
     1     'downbound particles.',
     1 /1X,'#  6nd and 7rd columns: PDF and STU for ',
     1     'absorbed particles.',
     1 /1X,'#  PDFs and STUs in units of 1/(cm*primary_particle).')
      DO K=1,NBTL
        XX=TLMIN+(K-0.5D0)*BSTL
        YERR1=3.0D0*SQRT(ABS(PDTL2(1,K)-PDTL(1,K)**2*DF))
        YAV1=PDTL(1,K)*DF*RBSTL
        YERR1=YERR1*DF*RBSTL
        YERR2=3.0D0*SQRT(ABS(PDTL2(2,K)-PDTL(2,K)**2*DF))
        YAV2=PDTL(2,K)*DF*RBSTL
        YERR2=YERR2*DF*RBSTL
        YERR3=3.0D0*SQRT(ABS(PDTL2(3,K)-PDTL(3,K)**2*DF))
        YAV3=PDTL(3,K)*DF*RBSTL
        YERR3=YERR3*DF*RBSTL
        WRITE(9,'(1P,E14.6,3(E14.6,E10.2))')
     1    XX,MAX(YAV1,1.0D-35),YERR1,
     1    MAX(YAV2,1.0D-35),YERR2,MAX(YAV3,1.0D-35),YERR3
      ENDDO
      CLOSE(9)
      ENDIF
C
C  ************  Energy and angular distributions of emerging particles.
C
      CALL ENANGW(SHN)  ! Energy and angular distributions.
C
C  ************  Positions of emerging particles.
C
      IF(LEMERG) THEN
        FACT=TWOPI*BSER
        OPEN(9,FILE='mapxy-up.dat')
        WRITE(9,9210) ZG(NLAY+1)
 9210   FORMAT(
     1  1X,'#  Results from PENCYL.',
     1 /1X,'#  Position distribution of emerging upbound particles.',
     1 /1X,'#  Coordinates (X,Y) at the upper plane, Z =',1P,
     1    E14.6,' cm',
     1 /1X,'#  1st and 2nd cols.: coordinates X and Y (cm).',
     1 /1X,'#  3rd and 4th cols.: PDF and STU for electrons (1/cm^2)',
     1 /1X,'#  5rd and 6th cols.: PDF and STU for photons (1/cm^2)',
     1 /1X,'#  7rd and 8th cols.: PDF and STU for positrons (1/cm^2)')
        PIXEL=BSEX*BSEY
        DO I=1,NPEX
          XX=XEL+(I-0.5D0)*BSEX
          DO J=1,NPEY
            YY=YEL+(J-0.5D0)*BSEY
            YERR1=3.0D0*SQRT(ABS(PEXY2(1,1,I,J)-PEXY(1,1,I,J)**2*DF))
            YAV1=PEXY(1,1,I,J)*DF/PIXEL
            YERR1=YERR1*DF/PIXEL
            YERR2=3.0D0*SQRT(ABS(PEXY2(1,2,I,J)-PEXY(1,2,I,J)**2*DF))
            YAV2=PEXY(1,2,I,J)*DF/PIXEL
            YERR2=YERR2*DF/PIXEL
            YERR3=3.0D0*SQRT(ABS(PEXY2(1,3,I,J)-PEXY(1,3,I,J)**2*DF))
            YAV3=PEXY(1,3,I,J)*DF/PIXEL
            YERR3=YERR3*DF/PIXEL
            WRITE(9,'(1P,2E14.6,3(E14.6,E10.2))')
     1        XX,YY,MAX(YAV1,1.0D-35),YERR1,
     1        MAX(YAV2,1.0D-35),YERR2,MAX(YAV3,1.0D-35),YERR3
          ENDDO
          WRITE(9,*) '   '
        ENDDO
        CLOSE(9)
C
        OPEN(9,FILE='mapr-up.dat')
        WRITE(9,9211) ZG(NLAY+1)
 9211   FORMAT(
     1  1X,'#  Results from PENCYL.',
     1 /1X,'#  Radial distribution of emerging upbound particles.',
     1 /1X,'#  Radial distance, R=SQRT(X^2+Y*2), at the upper plane,',
     1      ' Z =',1P,E14.6,' cm',
     1 /1X,'#  1st column: radial distance (cm).',
     1 /1X,'#  2nd and 3rd columns: PDF and STU for electrons (1/cm)',
     1 /1X,'#  4th and 5th columns: PDF and STU for photons (1/cm)',
     1 /1X,'#  6th and 7th columns: PDF and STU for positrons (1/cm)')
        DO I=1,NBRE
          R=(I-0.5D0)*BSER
          BSURF=FACT*R
          YERR1=3.0D0*SQRT(ABS(PER2(1,1,I)-PER(1,1,I)**2*DF))
          YAV1=PER(1,1,I)*DF/BSURF
          YERR1=YERR1*DF/BSURF
          YERR2=3.0D0*SQRT(ABS(PER2(1,2,I)-PER(1,2,I)**2*DF))
          YAV2=PER(1,2,I)*DF/BSURF
          YERR2=YERR2*DF/BSURF
          YERR3=3.0D0*SQRT(ABS(PER2(1,3,I)-PER(1,3,I)**2*DF))
          YAV3=PER(1,3,I)*DF/BSURF
          YERR3=YERR3*DF/BSURF
          WRITE(9,'(1P,E14.6,3(E14.6,E10.2))')
     1      R,MAX(YAV1,1.0D-35),YERR1,
     1      MAX(YAV2,1.0D-35),YERR2,MAX(YAV3,1.0D-35),YERR3
        ENDDO
        CLOSE(9)
C
        OPEN(9,FILE='mapxy-down.dat')
        WRITE(9,9212) ZG(1)
 9212   FORMAT(
     1  1X,'#  Results from PENCYL.',
     1 /1X,'#  Position distribution of emerging downbound particles.',
     1 /1X,'#  Coordinates (X,Y) at the lower plane, Z =',1P,
     1    E14.6,' cm',
     1 /1X,'#  1st and 2nd cols.: coordinates X and Y (cm).',
     1 /1X,'#  3rd and 4th cols.: PDF and STU for electrons (1/cm^2)',
     1 /1X,'#  5rd and 6th cols.: PDF and STU for photons (1/cm^2)',
     1 /1X,'#  7rd and 8th cols.: PDF and STU for positrons (1/cm^2)')
        PIXEL=BSEX*BSEY
        DO I=1,NPEX
          XX=XEL+(I-0.5D0)*BSEX
          DO J=1,NPEY
            YY=YEL+(J-0.5D0)*BSEY
            YERR1=3.0D0*SQRT(ABS(PEXY2(2,1,I,J)-PEXY(2,1,I,J)**2*DF))
            YAV1=PEXY(2,1,I,J)*DF/PIXEL
            YERR1=YERR1*DF/PIXEL
            YERR2=3.0D0*SQRT(ABS(PEXY2(2,2,I,J)-PEXY(2,2,I,J)**2*DF))
            YAV2=PEXY(2,2,I,J)*DF/PIXEL
            YERR2=YERR2*DF/PIXEL
            YERR3=3.0D0*SQRT(ABS(PEXY2(2,3,I,J)-PEXY(2,3,I,J)**2*DF))
            YAV3=PEXY(2,3,I,J)*DF/PIXEL
            YERR3=YERR3*DF/PIXEL
            WRITE(9,'(1P,2E14.6,3(E14.6,E10.2))')
     1        XX,YY,MAX(YAV1,1.0D-35),YERR1,
     1        MAX(YAV2,1.0D-35),YERR2,MAX(YAV3,1.0D-35),YERR3
          ENDDO
          WRITE(9,*) '   '
        ENDDO
        CLOSE(9)
C
        OPEN(9,FILE='mapr-down.dat')
        WRITE(9,9213) ZG(1)
 9213   FORMAT(
     1  1X,'#  Results from PENCYL.',
     1 /1X,'#  Radial distribution of emerging downbound particles.',
     1 /1X,'#  Radial distance, R=SQRT(X^2+Y*2), at the lower plane,',
     1      ' Z =',1P,E14.6,' cm',
     1 /1X,'#  1st column: radial distance (cm).',
     1 /1X,'#  2nd and 3rd columns: PDF and STU for electrons (1/cm)',
     1 /1X,'#  4th and 5th columns: PDF and STU for photons (1/cm)',
     1 /1X,'#  6th and 7th columns: PDF and STU for positrons (1/cm)')
        DO I=1,NBRE
          R=(I-0.5D0)*BSER
          BSURF=FACT*R
          YERR1=3.0D0*SQRT(ABS(PER2(2,1,I)-PER(2,1,I)**2*DF))
          YAV1=PER(2,1,I)*DF/BSURF
          YERR1=YERR1*DF/BSURF
          YERR2=3.0D0*SQRT(ABS(PER2(2,2,I)-PER(2,2,I)**2*DF))
          YAV2=PER(2,2,I)*DF/BSURF
          YERR2=YERR2*DF/BSURF
          YERR3=3.0D0*SQRT(ABS(PER2(2,3,I)-PER(2,3,I)**2*DF))
          YAV3=PER(2,3,I)*DF/BSURF
          YERR3=YERR3*DF/BSURF
          WRITE(9,'(1P,E14.6,3(E14.6,E10.2))')
     1      R,MAX(YAV1,1.0D-35),YERR1,
     1      MAX(YAV2,1.0D-35),YERR2,MAX(YAV3,1.0D-35),YERR3
        ENDDO
        CLOSE(9)
      ENDIF
C
C  ****  Output of energy-deposition detectors.
C
      IF(NED.GT.0) THEN
        CALL ENDETW(SHN,TSIM,27)
      ENDIF
C
C  ************  2D dose and deposited charge distributions.
C
      IF(NDOD.GT.0) THEN
        DO KI=1,NDOD
          WRITE(BUF2,'(I5)') 1000+KI
          OFILE='dose-charge-'//BUF2(4:5)//'.dat'
          OPEN(9,FILE=OFILE)
          KL=ILAY(KBDOD(KI))
          KC=ICYL(KBDOD(KI))
          WRITE(9,9160) KL,KC
 9160     FORMAT(
     1  1X,'#  Results from PENCYL. ',
     1 /1X,'#  2D dose and charge distribs. in body KL=',I3,', KC=',I3,
     1 /1X,'#  1st column: radius (=sqrt(X**2+Y**2), in cm ',
     1 /1X,'#  2nd column: Z (cm)',
     1 /1X,'#  3rd and 4th cols.: dose and STU',
     1    ' (eV/(g*primary_particle)).',
     1 /1X,'#  5th and 6th cols.: charge and STU',
     1    ' (e/(cm^3*primary_particle)).')
          FACT=TWOPI*BRDOD(KI)*BZDOD(KI)
          DO KR=1,NRDOD(KI)
            R=RG(KL,KC)+(KR-0.5D0)*BRDOD(KI)
            BINVOL=R*FACT
            DO KZ=1,NZDOD(KI)
              Z=ZG(KL)+(KZ-0.5D0)*BZDOD(KI)
              DERR=3.0D0*SQRT(ABS(DOSE2(KI,KZ,KR)-DOSE(KI,KZ,KR)**2*DF))
              DAV=DOSE(KI,KZ,KR)*DF/BINVOL
              DERR=DERR*DF/BINVOL
              CERR=3.0D0*SQRT(ABS(CHAR2(KI,KZ,KR)-CHAR(KI,KZ,KR)**2*DF))
              CAV=CHAR(KI,KZ,KR)*DF/BINVOL
              CERR=CERR*DF/BINVOL
              WRITE(9,'(1P,2E14.6,2(E14.6,E10.2))')
     1          R,Z,MAX(DAV,1.0D-35),
     1          MAX(DERR,1.0D-35),CAV,MAX(CERR,1.0D-35)
            ENDDO
            WRITE(9,*) '  '
          ENDDO
          CLOSE(9)
        ENDDO
      ENDIF
C
      WRITE(27,3041) ISEED1(IDMP),ISEED2(IDMP)
 3041 FORMAT(/3X,'Last random seeds = ',I10,' , ',I10)
      WRITE(27,'(/3X,72(''-''))')
      CLOSE(27)
C
      WRITE(6,3042) SHN
 3042 FORMAT(2X,'Number of simulated showers =',1P,E14.7)
      WRITE(6,'(2X,''***  END  ***'')')
      STOP
C
  300 CONTINUE
      WRITE(6,'(2X,''***  Inconsistent dump file format.'')')
      STOP
      END


C  *********************************************************************
C                       SUBROUTINE TENANG
C  *********************************************************************
      SUBROUTINE TENANG(IEXIT,N)
C
C  Tallies energy and angular distributions of emerging particles,
C  writes and loads dump files, accumulates dump files from different
C  runs, and writes results.
C
      USE TRACK_mod
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER*4 (I-N)
      PARAMETER (PI=3.1415926535897932D0, TWOPI=2.0D0*PI,
     1  RA2DE=180.0D0/PI, DE2RA=PI/180.0D0)
      PARAMETER (FSAFE=1.000000001D0)  ! Safety factor.
      PARAMETER (NBEM=1500)  ! Max. no. of energy bins.
      PARAMETER (NBTHM=1800,NBPHM=180)  ! Max. nos of angular bins.
      COMMON/CENANG/EL,EU,THL,THU,BSE,RBSE,BSTH,RBSTH,BSPH,RBSPH,
     1  PDE(3,2,NBEM),PDE2(3,2,NBEM),PDEP(3,2,NBEM),
     2  PDA(3,NBTHM,NBPHM),PDA2(3,NBTHM,NBPHM),PDAP(3,NBTHM,NBPHM),
     4  LPDE(3,2,NBEM),LPDA(3,NBTHM,NBPHM),NE,NTH,NPH,LLE,LLTH
C
C  ************  Score contributions of a new particle.
C
C  ****  Energy distribution of emerging particles.
      IF(LLE.EQ.1) THEN
        KEn=1.0D0+(LOG(E)-EL)*RBSE
      ELSE
        KEn=1.0D0+(E-EL)*RBSE
      ENDIF
      IF(KEn.GT.0.AND.KEn.LE.NE) THEN
        IF(N.NE.LPDE(KPAR,IEXIT,KEn)) THEN
          PDE(KPAR,IEXIT,KEn)=PDE(KPAR,IEXIT,KEn)+PDEP(KPAR,IEXIT,KEn)
          PDE2(KPAR,IEXIT,KEn)=
     1      PDE2(KPAR,IEXIT,KEn)+PDEP(KPAR,IEXIT,KEn)**2
          PDEP(KPAR,IEXIT,KEn)=WGHT
          LPDE(KPAR,IEXIT,KEn)=N
        ELSE
          PDEP(KPAR,IEXIT,KEn)=PDEP(KPAR,IEXIT,KEn)+WGHT
        ENDIF
      ENDIF
C  ****  Angular distribution of emerging particles.
      THETA=ACOS(W)
      IF(LLTH.EQ.1) THEN
        KTH=1.0D0+(LOG(MAX(THETA,1.0D-12)*RA2DE)-THL)*RBSTH
        IF(KTH.LT.1) KTH=1
      ELSE
        KTH=1.0D0+THETA*RA2DE*RBSTH
      ENDIF
      IF(ABS(U).GT.1.0D-16) THEN  ! Azimuthal bin number corrected.
         PHI=ATAN2(V,U)
      ELSE IF(ABS(V).GT.1.0D-16) THEN
         PHI=ATAN2(V,U)
      ELSE
         PHI=0.0D0
      ENDIF
      IF(PHI.LT.0.0D0) PHI=TWOPI+PHI
      KPH=1.0D0+PHI*RA2DE*RBSPH
      IF(N.NE.LPDA(KPAR,KTH,KPH)) THEN
        PDA(KPAR,KTH,KPH)=PDA(KPAR,KTH,KPH)+PDAP(KPAR,KTH,KPH)
        PDA2(KPAR,KTH,KPH)=PDA2(KPAR,KTH,KPH)+PDAP(KPAR,KTH,KPH)**2
        PDAP(KPAR,KTH,KPH)=WGHT
        LPDA(KPAR,KTH,KPH)=N
      ELSE
        PDAP(KPAR,KTH,KPH)=PDAP(KPAR,KTH,KPH)+WGHT
      ENDIF
      RETURN
C
C  ************  Initialise the histograms.
C  Logarithmic (NB.LT.0) or uniform (NB.GT.0) scale.
C
      ENTRY ENANG0(EMIN,EMAX,NBE,NBTH,NBPH)  !<<<<<<<<<<<<<<<<<<<<<<<<<<
C
        IF (ABS(NBE).GT.NBEM) THEN
          WRITE(6,*) 'TENANG: NBE is too large.'
          WRITE(6,*) 'TENANG: Set the parameter NBEM equal to ',
     1      ABS(NBE)
          STOP 'TENANG: NBE is too large.'
        ENDIF
C
        IF (ABS(NBTH).GT.NBTHM) THEN
          WRITE(6,*) 'TENANG: NBTH is too large.'
          WRITE(6,*) 'TENANG: Set the parameter NBTHM equal to ',
     1      ABS(NBTH)
          STOP 'TENANG: NBTH is too large.'
        ENDIF
C
        IF (NBPH.GT.NBPHM) THEN
          WRITE(6,*) 'TENANG: NBPH is too large.'
          WRITE(6,*) 'TENANG: Set the parameter NBPHM equal to ',NBPH
          STOP 'TENANG: NBPH is too large.'
        ENDIF
C
        IF(NBE.LT.0) THEN
          LLE=1
          EL=LOG(EMIN)
          EU=LOG(EMAX)
          NE=-NBE
        ELSE
          LLE=0
          EL=EMIN
          EU=EMAX
          NE=NBE
        ENDIF
        BSE=FSAFE*(EU-EL)/DBLE(NE)
        RBSE=1.0D0/BSE
C
        IF(NBTH.LT.0) THEN
          LLTH=1
          THL=LOG(1.0D-2)
          THU=LOG(180.0D0)
          NTH=-NBTH
        ELSE
          LLTH=0
          THL=0.0D0
          THU=180.0D0
          NTH=NBTH
        ENDIF
        BSTH=FSAFE*(THU-THL)/DBLE(NTH)
        RBSTH=1.0D0/BSTH
C
        IF(NBPH.LT.0) THEN
          NPH=-NBPH
        ELSE
          NPH=NBPH
        ENDIF
        BSPH=FSAFE*360.0D0/DBLE(NPH)
        RBSPH=1.0D0/BSPH
C
        DO I=1,3
          DO J=1,2
            DO K=1,NBEM
              PDE(I,J,K)=0.0D0
              PDE2(I,J,K)=0.0D0
              PDEP(I,J,K)=0.0D0
              LPDE(I,J,K)=0
            ENDDO
          ENDDO
        ENDDO
C
        DO I=1,3
          DO J=1,NBTHM
            DO K=1,NBPHM
              PDA(I,J,K)=0.0D0
              PDA2(I,J,K)=0.0D0
              PDAP(I,J,K)=0.0D0
              LPDA(I,J,K)=0
            ENDDO
          ENDDO
        ENDDO
      RETURN
C
C  ************  DUMP. Write the histograms.
C
      ENTRY ENANGD(IWR)  !<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
C
C  ****  Transfer partial counters to global counters.
        DO KP=1,3
          DO IEX=1,2
            DO KEn=1,NE
              PDE(KP,IEX,KEn)=PDE(KP,IEX,KEn)+PDEP(KP,IEX,KEn)
              PDE2(KP,IEX,KEn)=PDE2(KP,IEX,KEn)+PDEP(KP,IEX,KEn)**2
              PDEP(KP,IEX,KEn)=0.0D0
              LPDE(KP,IEX,KEn)=0
            ENDDO
          ENDDO
        ENDDO
C
        DO KP=1,3
          DO KTH=1,NTH
            DO KPH=1,NPH
              PDA(KP,KTH,KPH)=PDA(KP,KTH,KPH)+PDAP(KP,KTH,KPH)
              PDA2(KP,KTH,KPH)=PDA2(KP,KTH,KPH)+PDAP(KP,KTH,KPH)**2
              PDAP(KP,KTH,KPH)=0.0D0
              LPDA(KP,KTH,KPH)=0
            ENDDO
          ENDDO
        ENDDO
C  ****  Write parameters and counters.
        WRITE(IWR,*) NE,EL,EU,BSE,RBSE,LLE
        WRITE(IWR,*) (((PDE(I,J,K),K=1,NE),J=1,2),I=1,3)
        WRITE(IWR,*) (((PDE2(I,J,K),K=1,NE),J=1,2),I=1,3)
        WRITE(IWR,*) NTH,THL,THU,BSTH,RBSTH,NPH,BSPH,RBSPH,LLTH
        WRITE(IWR,*) (((PDA(I,J,K),K=1,NPH),J=1,NTH),I=1,3)
        WRITE(IWR,*) (((PDA2(I,J,K),K=1,NPH),J=1,NTH),I=1,3)
      RETURN
C
C  ************  RESUME. Read histograms from a previous run.
C
      ENTRY ENANGR(IRD)  !<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
C
        READ(IRD,*) NE,EL,EU,BSE,RBSE,LLE
        READ(IRD,*) (((PDE(I,J,K),K=1,NE),J=1,2),I=1,3)
        READ(IRD,*) (((PDE2(I,J,K),K=1,NE),J=1,2),I=1,3)
        READ(IRD,*) NTH,THL,THU,BSTH,RBSTH,NPH,BSPH,RBSPH,LLTH
        READ(IRD,*) (((PDA(I,J,K),K=1,NPH),J=1,NTH),I=1,3)
        READ(IRD,*) (((PDA2(I,J,K),K=1,NPH),J=1,NTH),I=1,3)
C
        DO I=1,3
          DO J=1,2
            DO K=1,NE
              PDEP(I,J,K)=0.0D0
              LPDE(I,J,K)=0
            ENDDO
          ENDDO
        ENDDO
C
        DO I=1,3
          DO J=1,NTH
            DO K=1,NPH
              PDAP(I,J,K)=0.0D0
              LPDA(I,J,K)=0
            ENDDO
          ENDDO
        ENDDO
      RETURN
C
C  ************  ADD results from previous runs.
C  Use ENANGR to load histograms from one of the runs and then call
C  ENANGA to accumulate results from other runs.
C
      ENTRY ENANGA(IRD)  !<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
C
        READ(IRD,*) NE,EL,EU,BSE,RBSE,LLE
        READ(IRD,*) (((PDEP(I,J,K),K=1,NE),J=1,2),I=1,3)
        DO I=1,3
          DO J=1,2
            DO K=1,NE
              PDE(I,J,K)=PDE(I,J,K)+PDEP(I,J,K)
            ENDDO
          ENDDO
        ENDDO
        READ(IRD,*) (((PDEP(I,J,K),K=1,NE),J=1,2),I=1,3)
        DO I=1,3
          DO J=1,2
            DO K=1,NE
              PDE2(I,J,K)=PDE2(I,J,K)+PDEP(I,J,K)
              PDEP(I,J,K)=0.0D0
            ENDDO
          ENDDO
        ENDDO
        READ(IRD,*) NTH,THL,THU,BSTH,RBSTH,NPH,BSPH,RBSPH,LLTH
        READ(IRD,*) (((PDAP(I,J,K),K=1,NPH),J=1,NTH),I=1,3)
        DO I=1,3
          DO J=1,NTH
            DO K=1,NPH
              PDA(I,J,K)=PDA(I,J,K)+PDAP(I,J,K)
            ENDDO
          ENDDO
        ENDDO
        READ(IRD,*) (((PDAP(I,J,K),K=1,NPH),J=1,NTH),I=1,3)
        DO I=1,3
          DO J=1,NTH
            DO K=1,NPH
              PDA2(I,J,K)=PDA2(I,J,K)+PDAP(I,J,K)
              PDAP(I,J,K)=0.0D0
            ENDDO
          ENDDO
        ENDDO
      RETURN
C
C  ************  Write simulation results in separate output files.
C
      ENTRY ENANGW(SHN)  !<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
C
C  ****  Transfer partial counters to global counters.
        DO KP=1,3
          DO IEX=1,2
            DO KEn=1,NE
              PDE(KP,IEX,KEn)=PDE(KP,IEX,KEn)+PDEP(KP,IEX,KEn)
              PDE2(KP,IEX,KEn)=PDE2(KP,IEX,KEn)+PDEP(KP,IEX,KEn)**2
              PDEP(KP,IEX,KEn)=0.0D0
              LPDE(KP,IEX,KEn)=0
            ENDDO
          ENDDO
        ENDDO
C
        DO KP=1,3
          DO KTH=1,NTH
            DO KPH=1,NPH
              PDA(KP,KTH,KPH)=PDA(KP,KTH,KPH)+PDAP(KP,KTH,KPH)
              PDA2(KP,KTH,KPH)=PDA2(KP,KTH,KPH)+PDAP(KP,KTH,KPH)**2
              PDAP(KP,KTH,KPH)=0.0D0
              LPDA(KP,KTH,KPH)=0
            ENDDO
          ENDDO
        ENDDO
C
        DF=1.0D0/SHN
C
C  ****  Energy distributions of emerging particles.
C
C  ----  Upbound particles.
        OPEN(9,FILE='energy-up.dat')
        WRITE(9,9110)
 9110   FORMAT(
     1    1X,'#  Results from PENCYL.',
     1   /1X,'#  Energy distributions of upbound particles.',
     1   /1X,'#  1st column: E (eV).',
     1   /1X,'#  2nd and 3rd columns: PDF and STU for electrons.',
     1   /1X,'#  4th and 5th columns: PDF and STU for photons.',
     1   /1X,'#  6th and 7th columns: PDF and STU for positrons.',
     1   /1X,'#    PDFs and STUs in units of 1/(eV*primary_particle).')
        DO KEn=1,NE
          IF(LLE.EQ.1) THEN
            XLOW=EXP(EL+(KEn-1)*BSE)
            XUPP=EXP(EL+KEn*BSE)
            XX=0.5D0*(XUPP+XLOW)
            BINS=XUPP-XLOW
          ELSE
            XX=EL+(KEn-0.5D0)*BSE
            BINS=BSE
          ENDIF
          YERR1=3.0D0*SQRT(ABS(PDE2(1,1,KEn)-PDE(1,1,KEn)**2*DF))
          YAV1=PDE(1,1,KEn)*DF/BINS
          YERR1=YERR1*DF/BINS
          YERR2=3.0D0*SQRT(ABS(PDE2(2,1,KEn)-PDE(2,1,KEn)**2*DF))
          YAV2=PDE(2,1,KEn)*DF/BINS
          YERR2=YERR2*DF/BINS
          YERR3=3.0D0*SQRT(ABS(PDE2(3,1,KEn)-PDE(3,1,KEn)**2*DF))
          YAV3=PDE(3,1,KEn)*DF/BINS
          YERR3=YERR3*DF/BINS
          WRITE(9,'(1P,E14.6,3(E14.6,E10.2))')
     1      XX,MAX(YAV1,1.0D-35),YERR1,
     1      MAX(YAV2,1.0D-35),YERR2,MAX(YAV3,1.0D-35),YERR3
        ENDDO
        CLOSE(9)
C  ----  Downbound particles.
        OPEN(9,FILE='energy-down.dat')
        WRITE(9,9120)
 9120   FORMAT(
     1    1X,'#  Results from PENCYL.',
     1   /1X,'#  Energy distributions of downbound particles.',
     1   /1X,'#  1st column: E (eV).',
     1   /1X,'#  2nd and 3rd columns: PDF and STU for electrons.',
     1   /1X,'#  4th and 5th columns: PDF and STU for photons.',
     1   /1X,'#  6th and 7th columns: PDF and STU for positrons.',
     1   /1X,'#    PDFs and STUs in units of 1/(eV*primary_particle).')
        DO KEn=1,NE
          IF(LLE.EQ.1) THEN
            XLOW=EXP(EL+(KEn-1)*BSE)
            XUPP=EXP(EL+KEn*BSE)
            XX=0.5D0*(XUPP+XLOW)
            BINS=XUPP-XLOW
          ELSE
            XX=EL+(KEn-0.5D0)*BSE
            BINS=BSE
          ENDIF
          YERR1=3.0D0*SQRT(ABS(PDE2(1,2,KEn)-PDE(1,2,KEn)**2*DF))
          YAV1=PDE(1,2,KEn)*DF/BINS
          YERR1=YERR1*DF/BINS
          YERR2=3.0D0*SQRT(ABS(PDE2(2,2,KEn)-PDE(2,2,KEn)**2*DF))
          YAV2=PDE(2,2,KEn)*DF/BINS
          YERR2=YERR2*DF/BINS
          YERR3=3.0D0*SQRT(ABS(PDE2(3,2,KEn)-PDE(3,2,KEn)**2*DF))
          YAV3=PDE(3,2,KEn)*DF/BINS
          YERR3=YERR3*DF/BINS
          WRITE(9,'(1P,E14.6,3(E14.6,E10.2))')
     1      XX,MAX(YAV1,1.0D-35),YERR1,
     1      MAX(YAV2,1.0D-35),YERR2,MAX(YAV3,1.0D-35),YERR3
        ENDDO
        CLOSE(9)
C
C  ************  Angular distributions of emerging particles.
C
        IF(NPH.GT.1) THEN
          OPEN(9,FILE='angle.dat')
          WRITE(9,9210)
 9210     FORMAT(
     1      1X,'#  Results from PENCYL.',
     1     /1X,'#  Angular distributions of emerging particles.',
     1     /1X,'#  1st and 2nd columns: THETA and PHI (deg).',
     1     /1X,'#  3rd and 4th columns: PDF and STU for electrons.',
     1     /1X,'#  5th and 6th columns: PDF and STU for photons.',
     1     /1X,'#  7th and 8th columns: PDF and STU for positrons.',
     1     /1X,'#  PDFs and STUs in units of 1/(sr*primary_particle).')
          DO KTH=1,NTH
            IF(LLTH.EQ.1) THEN
              XLOW=EXP(THL+(KTH-1)*BSTH)
              XUPP=EXP(THL+KTH*BSTH)
            ELSE
              XLOW=THL+(KTH-1)*BSTH
              XUPP=THL+KTH*BSTH
            ENDIF
            XX=0.5D0*(XUPP+XLOW)
            BINS=XUPP-XLOW
            DSANG=(COS(XLOW*DE2RA)-COS(XUPP*DE2RA))*(BSPH*DE2RA)
            DO L=1,NPH
              YY=(L-0.5D0)*BSPH
              YERR1=3.0D0*SQRT(ABS(PDA2(1,KTH,L)-PDA(1,KTH,L)**2*DF))
              YAV1=PDA(1,KTH,L)*DF/DSANG
              YERR1=YERR1*DF/DSANG
              YERR2=3.0D0*SQRT(ABS(PDA2(2,KTH,L)-PDA(2,KTH,L)**2*DF))
              YAV2=PDA(2,KTH,L)*DF/DSANG
              YERR2=YERR2*DF/DSANG
              YERR3=3.0D0*SQRT(ABS(PDA2(3,KTH,L)-PDA(3,KTH,L)**2*DF))
              YAV3=PDA(3,KTH,L)*DF/DSANG
              YERR3=YERR3*DF/DSANG
              WRITE(9,'(1P,2E14.6,3(E14.6,E10.2))')
     1          XX,YY,MAX(YAV1,1.0D-35),YERR1,
     1          MAX(YAV2,1.0D-35),YERR2,MAX(YAV3,1.0D-35),YERR3
            ENDDO
            WRITE(9,*) '   '
          ENDDO
          CLOSE(9)
        ENDIF
C
        OPEN(9,FILE='polar-angle.dat')
        WRITE(9,9310)
 9310   FORMAT(
     1    1X,'#  Results from PENCYL.',
     1   /1X,'#  Angular distributions of emerging particles.',
     1   /1X,'#  1st column: THETA (deg).',
     1   /1X,'#  2nd and 3rd columns: PDF and STU for electrons.',
     1   /1X,'#  4th and 5th columns: PDF and STU for photons.',
     1   /1X,'#  6th and 7th columns: PDF and STU for positrons',
     1   /1X,'#  PDFs and STUs in units of 1/(sr*primary_particle).')
        DO KTH=1,NTH
          IF(LLTH.EQ.1) THEN
            XLOW=EXP(THL+(KTH-1)*BSTH)
            XUPP=EXP(THL+KTH*BSTH)
          ELSE
            XLOW=THL+(KTH-1)*BSTH
            XUPP=THL+KTH*BSTH
          ENDIF
          XX=0.5D0*(XUPP+XLOW)
          BINS=XUPP-XLOW
          DSANG=(COS(XLOW*DE2RA)-COS(XUPP*DE2RA))*TWOPI
C
          S1=0.0D0
          S2=0.0D0
          DO L=1,NPH
            S1=S1+PDA(1,KTH,L)
            S2=S2+PDA2(1,KTH,L)
          ENDDO
          YERR1=3.0D0*SQRT(ABS(S2-S1**2*DF))
          YAV1=S1*DF/DSANG
          YERR1=YERR1*DF/DSANG
C
          S1=0.0D0
          S2=0.0D0
          DO L=1,NPH
            S1=S1+PDA(2,KTH,L)
            S2=S2+PDA2(2,KTH,L)
          ENDDO
          YERR2=3.0D0*SQRT(ABS(S2-S1**2*DF))
          YAV2=S1*DF/DSANG
          YERR2=YERR2*DF/DSANG
C
          S1=0.0D0
          S2=0.0D0
          DO L=1,NPH
            S1=S1+PDA(3,KTH,L)
            S2=S2+PDA2(3,KTH,L)
          ENDDO
          YERR3=3.0D0*SQRT(ABS(S2-S1**2*DF))
          YAV3=S1*DF/DSANG
          YERR3=YERR3*DF/DSANG
C
          WRITE(9,'(1P,E14.6,3(E14.6,E10.2))')
     1      XX,MAX(YAV1,1.0D-35),YERR1,
     1      MAX(YAV2,1.0D-35),YERR2,MAX(YAV3,1.0D-35),YERR3
        ENDDO
        CLOSE(9)
C
      RETURN
      END


C  *********************************************************************
C                       SUBROUTINE SENDET
C  *********************************************************************
      SUBROUTINE SENDET(ED,ID)
C
C  Tallies spectra from energy-deposition detectors, writes and loads
C  dump files, accumulates dump files from different runs, and writes
C  results.
C
      USE TRACK_mod
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER*4 (I-N)
      PARAMETER (FSAFE=1.000000001D0)  ! Safety factor.
C
      CHARACTER FNSPC*20
      PARAMETER (NIDM=25,NBEM=1000)
      CHARACTER SPCDEO(NIDM)*20
      SAVE SPCDEO
C
      DIMENSION AUX1(NIDM),AUX2(NIDM,NBEM)
      COMMON/CENDET/EL(NIDM),EU(NIDM),BSE(NIDM),RBSE(NIDM),
     1  EDEP(NIDM),EDEP2(NIDM),DET(NIDM,NBEM),
     1  NE(NIDM),LLE(NIDM),NID
C
      DATA NIDS/0/
      SAVE NIDS
C
C  ************  Deposited energy spectrum.
C                ED includes the particle weight.
C
        EDEP(ID)=EDEP(ID)+ED
        EDEP2(ID)=EDEP2(ID)+ED**2
C
        IF(ED.GT.1.0D-5) THEN
          IF(LLE(ID).EQ.1) THEN
            IE=1.0D0+(LOG(ED)-EL(ID))*RBSE(ID)
          ELSE
            IE=1.0D0+(ED-EL(ID))*RBSE(ID)
          ENDIF
          IF(IE.GT.0.AND.IE.LE.NE(ID)) THEN
            DET(ID,IE)=DET(ID,IE)+1.0D0
          ENDIF
        ENDIF
      RETURN
C
C  ************  Initialise a new detector.
C  Logarithmic (NB.LT.0) or uniform (NB.GT.0) scale.
C
      ENTRY ENDET0(EMIN,EMAX,NB,FNSPC,ID)  !<<<<<<<<<<<<<<<<<<<<<<<<<<<<
C
        IF(ID.LE.NIDS) THEN
          WRITE(26,*) 'SENDET: Detector already defined.',ID
          STOP 'SENDET: Detector cannot be defined.'
        ENDIF
        NIDS=ID
        NID=ID
C
        IF(NID.GT.NIDM) THEN
          WRITE(26,'(3X,''NID = '',I4)') NID
          WRITE(26,*) 'SENDET: Too many detectors.'
          STOP 'SENDET: Too many detectors.'
        ENDIF
C
        SPCDEO(ID)=FNSPC
C
        IF (ABS(NB).GT.NBEM) THEN
          WRITE(26,*) 'SENDET: NB is too large.'
          WRITE(26,*) 'SENDET: Set the parameter NBEM equal to ',
     1      ABS(NB)
          STOP 'SENDET: NB is too large.'
        ENDIF
C
        IF(NB.LT.0) THEN
          LLE(ID)=1
          EL(ID)=LOG(EMIN)
          EU(ID)=LOG(EMAX)
          NE(ID)=-NB
        ELSE
          LLE(ID)=0
          EL(ID)=EMIN
          EU(ID)=EMAX
          NE(ID)=NB
        ENDIF
        BSE(ID)=FSAFE*(EU(ID)-EL(ID))/DBLE(NE(ID))
        RBSE(ID)=1.0D0/BSE(ID)
C
        EDEP(ID)=0.0D0
        EDEP2(ID)=0.0D0
C
        DO J=1,NBEM
          DET(ID,J)=0.0D0
        ENDDO
      RETURN
C
C  ************  DUMP. Write the histograms.
C
      ENTRY ENDETD(IWR)  !<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
C
        WRITE(IWR,*) NID
        WRITE(IWR,*) (EDEP(I),I=1,NID)
        WRITE(IWR,*) (EDEP2(I),I=1,NID)
        DO I=1,NID
          WRITE(IWR,'(A)') SPCDEO(I)
          WRITE(IWR,*) NE(I),EL(I),EU(I),BSE(I),RBSE(I),LLE(I)
          WRITE(IWR,*) (DET(I,J),J=1,NE(I))
        ENDDO
      RETURN
C
C  ************  RESUME. Read histograms from a previous run.
C
      ENTRY ENDETR(IRD)  !<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
C
        READ(IRD,*) NID
        READ(IRD,*) (EDEP(I),I=1,NID)
        READ(IRD,*) (EDEP2(I),I=1,NID)
        DO I=1,NID
          READ(IRD,'(A)') SPCDEO(I)
          READ(IRD,*) NE(I),EL(I),EU(I),BSE(I),RBSE(I),LLE(I)
          READ(IRD,*) (DET(I,J),J=1,NE(I))
        ENDDO
      RETURN
C
C  ************  ADD results from previous runs.
C  Use ENDETR to load histograms from one of the runs and then call
C  ENDETA to accumulate results from other runs.
C
      ENTRY ENDETA(IRD)  !<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
C
        READ(IRD,*) NID
        READ(IRD,*) (AUX1(I),I=1,NID)
        DO I=1,NID
          EDEP(I)=EDEP(I)+AUX1(I)
        ENDDO
        READ(IRD,*) (AUX1(I),I=1,NID)
        DO I=1,NID
          EDEP2(I)=EDEP2(I)+AUX1(I)
        ENDDO
        DO I=1,NID
          READ(IRD,'(A)') SPCDEO(I)
          READ(IRD,*) NE(I),EL(I),EU(I),BSE(I),RBSE(I),LLE(I)
          READ(IRD,*) (AUX2(I,J),J=1,NE(I))
          DO J=1,NE(I)
            DET(I,J)=DET(I,J)+AUX2(I,J)
          ENDDO
        ENDDO
      RETURN
C
C  ************  Write simulation results in separate output files.
C
      ENTRY ENDETW(SHN,TSIM,IWR)  !<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
C
      IF(NID.EQ.0) RETURN
      DF=1.0D0/SHN
C
C  ****  Average deposited energies in energy-deposition detectors.
C
      WRITE(IWR,3015)
 3015 FORMAT(/3X,'Average deposited energies (energy detectors):')
      DO I=1,NID
        QER=3.0D0*DF*SQRT(ABS(EDEP2(I)-EDEP(I)**2*DF))
        QAV=EDEP(I)*DF
        IF(QER.GT.1.0D-10*ABS(QAV)) THEN
          EFFIC=QAV**2/((QER/3.0D0)**2*TSIM)
        ELSE
          EFFIC=0.0D0
        ENDIF
        WRITE(IWR,3014) I,QAV,QER,EFFIC
      ENDDO
 3014 FORMAT(6X,'Detector #',I2,' ... ',1P,E13.6,' +-',E8.1,' eV',4X,
     1  '(effic. =',E9.2,')')
C
C  ****  Detector efficiencies.
C
      WRITE(IWR,3018)
 3018 FORMAT(/3X,'Spectral areas (energy-deposition detectors):')
      DO I=1,NID
        SUM=0.0D0
        ESUM=0.0D0
        DO J=1,NE(I)
          YAV=DET(I,J)*DF
          YERR=3.0D0*SQRT(ABS(YAV*(1.0D0-YAV))*DF)
          SUM=SUM+YAV
          ESUM=ESUM+YERR
        ENDDO
        WRITE(IWR,3017) I,SUM,ESUM
      ENDDO
 3017 FORMAT(6X,'Detector #',I2,' ... ',1P,E13.6,' +-',E8.1)
C
C  ************  Spectra from energy-deposition detectors.
C
      DO I=1,NID
        OPEN(9,FILE=SPCDEO(I))
        WRITE(9,9310) I
 9310 FORMAT(
     1  1X,'#  Results from PENCYL. Output from energy-deposition',
     1  ' detector #',I3,
     1 /1X,'#  Deposited energy spectrum.',
     1 /1X,'#  WARNING: May be strongly biased if interaction ',
     1  'forcing is used!',
     1 /1X,'#  1st column: deposited energy (eV).',
     1 /1X,'#  2nd column: probability density (1/(eV*particle)).',
     1 /1X,'#  3rd column: statistical uncertainty (3 sigma).')
        DO J=1,NE(I)
          IF(LLE(I).EQ.1) THEN
            XLOW=EXP(EL(I)+(J-1)*BSE(I))
            XUPP=EXP(EL(I)+J*BSE(I))
            XX=0.5D0*(XUPP+XLOW)
            BINS=XUPP-XLOW
          ELSE
            XX=EL(I)+(J-0.5D0)*BSE(I)
            BINS=BSE(I)
          ENDIF
          YAV=DET(I,J)*DF
          YERR=3.0D0*SQRT(ABS(YAV*(1.0D0-YAV))*DF)
          YAV=YAV/BINS
          YERR=YERR/BINS
          WRITE(9,'(1P,2E14.6,E10.2)') XX,MAX(YAV,1.0D-35),YERR
        ENDDO
        CLOSE(9)
      ENDDO
C
      RETURN
      END
