# Gnuplot version 5.0 patchlevel 3.
# Plots results from 'penmain.f'

#-----------------------------------------------------------------------

reset

# Define the appropriate terminal
 set terminal wxt size 900,600 enhanced font 'Arial,12' dashlength 2.0
#set terminal windows size 900,600 enhanced font 'Arial,12'
#set terminal qt size 900,600 font 'Arial,12' enhanced

set encoding utf8  # Character encoding
set mouse
set zero 1.0e-99

# Soft colors
#set linetype  1 lw 2 lc rgb 29390     # UB blue
#set linetype  2 lw 2 lc rgb 13382400  # soft red
#set linetype  3 lw 2 lc rgb 39168     # dark green
#set linetype  4 lw 2 lc rgb 10053171  # brown
#set linetype  5 lw 2 lc rgb 16724940  # pink

set linetype  1 lw 2 lc rgb 255       # blue
set linetype  2 lw 2 lc rgb 16711680  # red
set linetype  3 lw 2 lc rgb 65280     # green
set linetype  4 lw 2 lc rgb 10053171  # brown
set linetype  5 lw 2 lc rgb 16711935  # magenta

set linetype 10 lw 2 lc rgb 0         # black
set linetype 11 lw 2 lc rgb 8421504   # 50 percent black
set xzeroaxis linestyle 11
set yzeroaxis linestyle 11

#set palette rgbformulae 7,5,15
set palette defined ( 1 "blue", 2 "green", 3 "yellow" , 4 "orange", 5 "red")
#set palette defined ( 1 "blue", 2 "green", 3 "yellow" , 4 "white")
#set palette defined ( 1 "red", 2 "orange", 3 "yellow" , 4 "white")
set linetype 100 lw 1 lc rgb 0         # black

set dashtype 1 (8,3)       # dashed
set dashtype 2 (12,3,1,3)  # dot-dashed
set dashtype 3 (4,3)       # dotted

set tmargin at screen 0.90
set bmargin at screen 0.13
set rmargin at screen 0.94
set lmargin at screen 0.15

set size 1,1  # The plot fills the available canvas
set border lw 2.0
set pointsize 0.8
set bars 0.5

#set format xy "10^{%T}"
#set format xy "%.1te%+-3T"
#set format cb "%.1te%+-3T"

#-----------------------------------------------------------------------

set format xy "%.1te%+-3T"
set ylabel 'P(E)   (1/eV)'
set xlabel 'E   (eV)'

set xrange [*:*]
set nologscale x

set yrange [1.e-14:*]
set logscale y

set mxtics 10
set mytics 10
set grid xtics ytics mxtics mytics ls 11 lw 1, ls 11 lw 1
unset grid

set title 'Energy spectrum of angular detector #1'
plot 'spc-angdet-01.dat' u 1:2 t 'electrons' w histeps ls 1,\
     'spc-angdet-01.dat' u 1:2:3 not w errorbars ls 1,\
     'spc-angdet-01.dat' u 1:4 t 'photons' w histeps ls 2,\
     'spc-angdet-01.dat' u 1:4:5 not w errorbars ls 2,\
     'spc-angdet-01.dat' u 1:6 t 'positrons' w histeps ls 3,\
     'spc-angdet-01.dat' u 1:6:7 not w errorbars ls 3
pause -1 'Press enter to continue'

set title 'Energy spectrum of angular detector #2'
plot 'spc-angdet-02.dat' u 1:2 t 'electrons' w histeps ls 1,\
     'spc-angdet-02.dat' u 1:2:3 not w errorbars ls 1,\
     'spc-angdet-02.dat' u 1:4 t 'photons' w histeps ls 2,\
     'spc-angdet-02.dat' u 1:4:5 not w errorbars ls 2,\
     'spc-angdet-02.dat' u 1:6 t 'positrons' w histeps ls 3,\
     'spc-angdet-02.dat' u 1:6:7 not w errorbars ls 3
pause -1 'Press enter to continue'

set title 'Energy spectrum of angular detector #3'
plot 'spc-angdet-03.dat' u 1:2 t 'electrons' w histeps ls 1,\
     'spc-angdet-03.dat' u 1:2:3 not w errorbars ls 1,\
     'spc-angdet-03.dat' u 1:4 t 'photons' w histeps ls 2,\
     'spc-angdet-03.dat' u 1:4:5 not w errorbars ls 2,\
     'spc-angdet-03.dat' u 1:6 t 'positrons' w histeps ls 3,\
     'spc-angdet-03.dat' u 1:6:7 not w errorbars ls 3
pause -1 'Press enter to continue'

set title 'Energy spectrum of angular detector #4'
plot 'spc-angdet-04.dat' u 1:2 t 'electrons' w histeps ls 1,\
     'spc-angdet-04.dat' u 1:2:3 not w errorbars ls 1,\
     'spc-angdet-04.dat' u 1:4 t 'photons' w histeps ls 2,\
     'spc-angdet-04.dat' u 1:4:5 not w errorbars ls 2,\
     'spc-angdet-04.dat' u 1:6 t 'positrons' w histeps ls 3,\
     'spc-angdet-04.dat' u 1:6:7 not w errorbars ls 3
pause -1 'Press enter to continue'

set title 'Energy spectrum of angular detector #5'
plot 'spc-angdet-05.dat' u 1:2 t 'electrons' w histeps ls 1,\
     'spc-angdet-05.dat' u 1:2:3 not w errorbars ls 1,\
     'spc-angdet-05.dat' u 1:4 t 'photons' w histeps ls 2,\
     'spc-angdet-05.dat' u 1:4:5 not w errorbars ls 2,\
     'spc-angdet-05.dat' u 1:6 t 'positrons' w histeps ls 3,\
     'spc-angdet-05.dat' u 1:6:7 not w errorbars ls 3
pause -1 'Press enter to continue'

set title 'Energy spectrum of angular detector #6'
plot 'spc-angdet-06.dat' u 1:2 t 'electrons' w histeps ls 1,\
     'spc-angdet-06.dat' u 1:2:3 not w errorbars ls 1,\
     'spc-angdet-06.dat' u 1:4 t 'photons' w histeps ls 2,\
     'spc-angdet-06.dat' u 1:4:5 not w errorbars ls 2,\
     'spc-angdet-06.dat' u 1:6 t 'positrons' w histeps ls 3,\
     'spc-angdet-06.dat' u 1:6:7 not w errorbars ls 3
pause -1 'Press enter to continue'

set title 'Energy spectrum of angular detector #7'
plot 'spc-angdet-07.dat' u 1:2 t 'electrons' w histeps ls 1,\
     'spc-angdet-07.dat' u 1:2:3 not w errorbars ls 1,\
     'spc-angdet-07.dat' u 1:4 t 'photons' w histeps ls 2,\
     'spc-angdet-07.dat' u 1:4:5 not w errorbars ls 2,\
     'spc-angdet-07.dat' u 1:6 t 'positrons' w histeps ls 3,\
     'spc-angdet-07.dat' u 1:6:7 not w errorbars ls 3
pause -1 'Press enter to continue'

set title 'Energy spectrum of angular detector #8'
plot 'spc-angdet-08.dat' u 1:2 t 'electrons' w histeps ls 1,\
     'spc-angdet-08.dat' u 1:2:3 not w errorbars ls 1,\
     'spc-angdet-08.dat' u 1:4 t 'photons' w histeps ls 2,\
     'spc-angdet-08.dat' u 1:4:5 not w errorbars ls 2,\
     'spc-angdet-08.dat' u 1:6 t 'positrons' w histeps ls 3,\
     'spc-angdet-08.dat' u 1:6:7 not w errorbars ls 3
pause -1 'Press enter to continue'

set title 'Energy spectrum of angular detector #9'
plot 'spc-angdet-09.dat' u 1:2 t 'electrons' w histeps ls 1,\
     'spc-angdet-09.dat' u 1:2:3 not w errorbars ls 1,\
     'spc-angdet-09.dat' u 1:4 t 'photons' w histeps ls 2,\
     'spc-angdet-09.dat' u 1:4:5 not w errorbars ls 2,\
     'spc-angdet-09.dat' u 1:6 t 'positrons' w histeps ls 3,\
     'spc-angdet-09.dat' u 1:6:7 not w errorbars ls 3
pause -1 'Press enter to continue'

set title 'Energy spectrum of angular detector #10'
plot 'spc-angdet-10.dat' u 1:2 t 'electrons' w histeps ls 1,\
     'spc-angdet-10.dat' u 1:2:3 not w errorbars ls 1,\
     'spc-angdet-10.dat' u 1:4 t 'photons' w histeps ls 2,\
     'spc-angdet-10.dat' u 1:4:5 not w errorbars ls 2,\
     'spc-angdet-10.dat' u 1:6 t 'positrons' w histeps ls 3,\
     'spc-angdet-10.dat' u 1:6:7 not w errorbars ls 3
pause -1 'Press enter to continue'

