C  ****  Files included to simplify compilation.
      INCLUDE 'penelope.f'
      INCLUDE 'rita.f'
      INCLUDE 'timer.f'
C
C     PROGRAM LEA
C
C  This program computes the Linear Energy-Absorption (LEA) for electr-
C  ons, photons, and positrons by direct simulation with PENELOPE. The
C  adopted energy cutoffs, Delta_P (P=el, ph, po), are defined by the
C  user.
C
C  For photons, the mass energy-absorption coefficient, mu_en/rho, is
C  equal to LEA0/(rho*E), where E is the photon energy, rho is the mass
C  density of the material, and LEA0 is the LEA calculated with the
C  energy cutoffs Delta(ph)=0 and Delta(el)=Delta(po)=infinity.
C
C                                           Francesc Salvat. July 2017.
C
      USE TRACK_mod
      USE PENELOPE_mod
      USE PENERROR_mod
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER*4 (I-N)
      CHARACTER*20 PMFILE,MFNAME
      PARAMETER (REV=5.10998928D5)  ! Electron rest energy (eV)
      PARAMETER (TREV=2.0D0*REV)
C
      COMMON/RSEED/ISEED1,ISEED2
C
      DIMENSION PMFILE(MAXMAT)
C  ****  Current IMFPs.
      COMMON/CJUMP0/P(8),ST,DST,DSR,W1,W2,T1,T2
      COMMON/CJUMP1/ELAST1,ELAST2,MODE,KSOFTE,KSOFTI,KDELTA
C  ****  Composition data.
      COMMON/COMPOS/STF(MAXMAT,30),ZT(MAXMAT),AT(MAXMAT),RHO(MAXMAT),
     1  VMOL(MAXMAT),IZ(MAXMAT,30),NELEM(MAXMAT)
C  ****  Energy grid and interpolation constants for the current energy.
      COMMON/CEGRID/EMIN,EL,EU,ET(NEGP),DLEMP(NEGP),DLEMP1,DLFC,
     1  XEL,XE,XEK,KE
C  ****  Electron and positron bremsstrahlung yields.
      COMMON/CBRYLD/EBRY(MAXMAT,NEGP),PBRY(MAXMAT,NEGP)
      DIMENSION DEL(3)
C
      PARAMETER (NDIM=1500)
      DIMENSION ER(NDIM)
      IETAB=0
      NETAB=0
C
      ISEED1=1; ISEED2=1
      OPEN(26,FILE='LEA-tot.dat')  ! PENELOPE default output unit.
      WRITE(26,'(''#  Linear Energy Absorption (LEA) from PENELOPE'')')
C  ****  Material data file.
      WRITE(6,*) '  '
      WRITE(6,*) '  Enter the name of the material data file...'
      READ(5,'(A20)') MFNAME
      WRITE(6,'(''#  Material data file:  '', A24)') MFNAME
      WRITE(26,'(''#  Material data file:  '', A24)') MFNAME
C  ****  Primary particle.
      WRITE(6,*) '  '
      WRITE(6,*)
     1  '  Primary particle? (1=electron, 2=photon, 3=positron)'
      READ(5,*) KPARP
      IF(KPARP.EQ.1) THEN
        WRITE(6,'(''#  Primary particles: electrons'')')
        WRITE(26,'(''#  Primary particles: electrons'')')
      ELSE IF(KPARP.EQ.2) THEN
        WRITE(6,'(''#  Primary particles: photons'')')
        WRITE(26,'(''#  Primary particles: photons'')')
      ELSE IF(KPARP.EQ.3) THEN
        WRITE(6,'(''#  Primary particles: positrons'')')
        WRITE(26,'(''#  Primary particles: positrons'')')
      ELSE
        WRITE(6,'(A,I6)') 'KPARP =',KPARP
        STOP 'Incorrect particle type.'
      ENDIF
C
C  ****  Simulation parameters and cutoff energies.
C
      WRITE(6,*) '  Initialising PENELOPE ...'
      NMAT=1
      PMFILE(1)=MFNAME
      EABS(1,1)=50.0D0; EABS(2,1)=50.0D0; EABS(3,1)=50.0D0
      C1(1)=0.2D0; C2(1)=0.2D0
      WCC(1)=50.0D0; WCR(1)=50.0D0
      EPMAX=1.0D9
C  ****  Cutoff energies.
      DEL(1)=1.0D35
      DEL(2)=50.0D0
      DEL(3)=1.0D35
      WRITE(6,*) '  '
      WRITE(6,*) '  Enter the cutoff energies Delta_P (in eV)...'
      READ(5,*,ERR=30) DEL1,DEL2,DEL3
      DEL(1)=MAX(DEL1,EABS(1,1))
      DEL(2)=MAX(DEL2,EABS(2,1))
      DEL(3)=MAX(DEL3,EABS(3,1))
      WRITE(6,'(A,1P,E13.6,A)') '#  DELTA(el)=',DEL(1),' eV'
      WRITE(6,'(A,1P,E13.6,A)') '#  DELTA(ph)=',DEL(2),' eV'
      WRITE(6,'(A,1P,E13.6,A)') '#  DELTA(po)=',DEL(3),' eV'
      WRITE(26,'(A,1P,E13.6,A)') '#  DELTA(el)=',DEL(1),' eV'
      WRITE(26,'(A,1P,E13.6,A)') '#  DELTA(ph)=',DEL(2),' eV'
      WRITE(26,'(A,1P,E13.6,A)') '#  DELTA(po)=',DEL(3),' eV'
      WCC(1)=DEL(1)
      WCR(1)=DEL(2)
C
      IWR=12
      OPEN(IWR,FILE='material.dat')
      INFO=3
      CALL PEINIT(EPMAX,NMAT,IWR,INFO,PMFILE)
      IF(IRETRN.NE.0) STOP
      CLOSE(IWR)
      CALL ETABLE(ER,NPHD,1)
C
      FG=1.0D0  ! To prevent compilation warnings.
C
C  ****  Tolerance. The simulation stops when the statistical
C  uncertainty of the calculated LEA value (1-sigma) is less than TOL.
C
      WRITE(6,*) '  '
      WRITE(6,*) '  Tolerance?  (in %, typically 0.1)'
      READ(5,*) TOL
      IF(TOL.GT.1.0D0) TOL=1.0D0
      WRITE(6,'(A,1P,E13.6,A)') '#  Tolerance =',TOL,' %'
      WRITE(26,'(A,1P,E13.6,A)') '#  Tolerance =',TOL,' %'
C
C  ****  Allotted simulation time (in seconds), for each energy.
C
      WRITE(6,*) '  '
      WRITE(6,*) '  Allotted simulation time (in seconds)?'
      READ(5,*) TSECA
      IF(TSECA.LT.0.0D0) TSECA=900.0D0
      WRITE(6,'(A,1P,E13.6,A)') '#  Allotted simulation time =',
     1   TSECA,' s'
      WRITE(26,'(A,1P,E13.6,A)') '#  Allotted simulation time =',
     1   TSECA,' s'
C
      DUMP=10.0D0
      WRITE(26,'(A,1P,E13.6,A)') '#  Dump period=',DUMP,' s'
C
C  ****  Output file with tables of LEA and MEA.
C
      OPEN(27,FILE='LEA.dat')
      WRITE(27,'(''#  Energy Absorption (LEA and MEA) from PENELOPE'')')
      WRITE(27,'(''#  Material data file:  '', A24)') MFNAME
      WRITE(27,'(A,1P,E13.6,A)') '#  Mass density=',RHO(1),' g/cm**3'
      WRITE(27,'(A)') '# '
      IF(KPARP.EQ.1) WRITE(27,'(''#  Primary particles: electrons'')')
      IF(KPARP.EQ.2) WRITE(27,'(''#  Primary particles: photons'')')
      IF(KPARP.EQ.3) WRITE(27,'(''#  Primary particles: positrons'')')
      WRITE(27,'(A,1P,E13.6,A)') '#  DELTA(el)=',DEL(1),' eV'
      WRITE(27,'(A,1P,E13.6,A)') '#  DELTA(ph)=',DEL(2),' eV'
      WRITE(27,'(A,1P,E13.6,A)') '#  DELTA(po)=',DEL(3),' eV'
      WRITE(27,'(A)') '# '
      WRITE(27,'(A,1P,E13.6,A)') '#  Tolerance=',TOL,' %'
      WRITE(27,'(A,1P,E13.6,A)') '#  Allotted simulation time =',
     1   TSECA,' s'
      WRITE(27,'(A,1P,E13.6,A)') '#  Dump period=',DUMP,' s'
      WRITE(27,'(A)') '# '
      IF(KPARP.EQ.1.OR.KPARP.EQ.3) THEN
        WRITE(27,'(A)') '#  1 mtu = 1 g/cm**2'
        WRITE(27,'(A)') '# '
        WRITE(27,901)
  901   FORMAT('#',4X,'Energy',9X,'LEA',10X,'MEA',9X,'RMSTP',6X,
     1    'MEA/RMSTP',4X,'unc.',8X,'T',/,'#',5X,'(eV)',8X,'(eV/cm)',
     1     5X,'(eV/mtu)',5X,'(eV/mtu)',19X,'(%)',7X,'(s)',/,'# ',
     1     86('-'))
      ELSE IF(KPARP.EQ.2) THEN
        WRITE(27,'(A)') '#  F_g = LEA/(E*mu); MEA=F_g*E*mu/rho'
        WRITE(27,'(A)') '#  1 mtu = 1 g/cm**2'
        WRITE(27,'(A)') '# '
        WRITE(27,902)
  902   FORMAT('#',4X,'Energy',9X,'LEA',9X,'MEA/E',7X,'mu/rho',9X,'F_g',
     1    6X,'uncert.',6X,'T',/,'#',5X,'(eV)',8X,'(eV/cm)',6X,'(1/mtu)',
     1    6X,'(1/mtu)',19X,'(%)',7X,'(s)',/,'# ',86('-'))
      ENDIF
C
C  ****  Radiant energy.
C
      WRITE(6,*) '  '
      WRITE(6,'(3X,2A)') 'TRICK: to use the default energy grid, enter',
     1  ' a value of the'
      WRITE(6,'(10X,2A)') 'particle energy, E0, less than zero. The ',
     1  'calculation will'
      WRITE(6,'(10X,2A)') 'start at an energy greater than or equal',
     1  ' to ABS(E0).'
 10   CONTINUE
      WRITE(6,*) '  '
      WRITE(6,*) '  Radiant energy (in eV)?'
      READ(5,*,ERR=90,END=90) E0
      IF(E0.LT.0.0D0) THEN
        IETAB=1
        ESTART=MAX(ABS(E0),5.0D1)
 20     CONTINUE
        NETAB=NETAB+1
        IF(NETAB.GT.NPHD) GO TO 90
        E0=ER(NETAB)
        IF(E0.LT.ESTART) GO TO 20
      ENDIF
      WRITE(6,'(/,A,1P,E13.6,A)') '#  E0=',E0,' eV'
      WRITE(26,'(/,A,1P,E13.6,A)') '#  E0=',E0,' eV'
C
 30   CONTINUE
      IF(IETAB.EQ.1) THEN
        IF(NETAB.GT.1) THEN
          IF(ABS(E0-ER(NETAB-1)).LT.1.0D-12*E0) E0=E0+1.0D-5
        ENDIF
        IF(NETAB.LT.NPHD) THEN
          IF(ABS(E0-ER(NETAB+1)).LT.1.0D-12*E0) E0=E0-1.0D-5
        ENDIF
      ENDIF
C
      IF(KPARP.NE.1) THEN
        DO I=NEGP,1,-1
          IF(EXP(EBRY(1,I))*ET(I).LT.1.0D-4*E0) THEN
            EABS(1,1)=MAX(ET(I),DEL(1))
            GO TO 40
          ENDIF
        ENDDO
  40    CONTINUE
      ELSE
        EABS(1,1)=DEL(1)
      ENDIF
C
      EABS(2,1)=DEL(2)
C
      IF(KPARP.NE.3) THEN
        EABS(3,1)=MIN(MAX(1.0D-3*E0,5.0D3),EABS(1,1),DEL(3))
      ELSE
        EABS(3,1)=DEL(3)
      ENDIF
C
      WRITE(6,'(A,1P,E13.6,A)') '#  EABS(1)=',EABS(1,1),' eV'
      WRITE(6,'(A,1P,E13.6,A)') '#  EABS(2)=',EABS(2,1),' eV'
      WRITE(6,'(A,1P,E13.6,A)') '#  EABS(3)=',EABS(3,1),' eV'
      WRITE(26,'(A,1P,E13.6,A)') '#  EABS(1)=',EABS(1,1),' eV'
      WRITE(26,'(A,1P,E13.6,A)') '#  EABS(2)=',EABS(2,1),' eV'
      WRITE(26,'(A,1P,E13.6,A)') '#  EABS(3)=',EABS(3,1),' eV'
      DSMAX=1.0D20
C
C  ************  Simulation of interaction showers.
C
      NTOT=1000000000
      WGHT=1.0D0
      CALL TIME0
      MAT=1
      IBODY=1
      TMIN=DUMP
C
      IF(KPARP.EQ.2) GO TO 200
      IF(KPARP.EQ.3) GO TO 300
C
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C  ***********  Electrons.
C
C  ****  Soft stopping power and total cross section for hard energy-
C        loss interactions.
      CALL EINaT(E0,DEL(1),XH0,XH1,XH2,XS0,XS1,XS2,XT1,XT2,DELTA,1)
      CALL EBRaT(E0,DEL(1),RH0,RH1,RH2,RS1,RS2,1)
      SIGTN=VMOL(1)*(XH0+RH0)/RHO(1)
      SOFTS=VMOL(1)*(XS1+RS1)/RHO(1)
      STP=VMOL(1)*(XS1+XH1+RS1+RH1)/RHO(1)
      WRITE(6,'(A,1P,E13.6,A)') '#  HardTCS=',SIGTN,' cm**2/g'
      WRITE(6,'(A,1P,E13.6,A)') '#  SoftSTP=',SOFTS,' eV*cm**2/g'
      WRITE(6,'(A,1P,E13.6,A)') '#  STP=',STP,' eV*cm**2/g'
      WRITE(26,'(A,1P,E13.6,A)') '#  HardTCS=',SIGTN,' cm**2/g'
      WRITE(26,'(A,1P,E13.6,A)') '#  SoftSTP=',SOFTS,' eV*cm**2/g'
      WRITE(26,'(A,1P,E13.6,A)') '#  STP=',STP,' eV*cm**2/g'
      WRITE(26,'(A)') '#  LEA/rho in eV*cm**2/gr'
C
      IF(E0.LT.EABS(1,1)) THEN
        PERR=0.0D0
        RMEA=SOFTS
        RLEA=SOFTS*RHO(1)
        WRITE(27,'(1X,1P,5E13.5,E9.1,E12.4)')
     1    E0,RLEA,RMEA,SOFTS,RMEA/SOFTS,PERR,0.0D0
        IF(IETAB.EQ.1) THEN
          NETAB=NETAB+1
          IF(NETAB.GT.NPHD) GO TO 90
          E0=ER(NETAB)
          WRITE(6,'(/,A,1P,E13.6,A)') '#  E0=',E0,' eV'
          WRITE(26,'(/,A,1P,E13.6,A)') '#  E0=',E0,' eV'
          GO TO 30
        ELSE
          GO TO 10
        ENDIF
      ELSE
        WRITE(6,1000)
        WRITE(26,1000)
 1000   FORMAT(/,'#',6X,'N',10X,'LEA/rho',6X,'MEA/STP',5X,'RSTP/STP',
     1    4X,'unc.(%)',5X,'T(s)')
      ENDIF
      N=0; TDUMP=DUMP
      EDPT=0.0D0; EDPT2=0.0D0
C
C  ------------------------  Shower simulation starts here.
  101 N=N+1
      CALL CLEANS  ! Cleans the secondary stack.
C  ****  Initial state of a primary particle.
      KPAR=1
      E=E0
      EDP=E0
      IF(E.LT.EABS(1,1)) GO TO 104
      X=0.0D0; Y=0.0D0; Z=0.0D0
      U=0.0D0; V=0.0D0; W=1.0D0
      ILB(1)=1; ILB(2)=0; ILB(3)=0; ILB(4)=0; ILB(5)=0
      CALL START
  110 CONTINUE
      CALL JUMP(DSMAX,DS)  ! Path length next interaction.
      MODE=1; P(2)=0.0D0; P(6)=0.0D0; KSOFTI=0; KDELTA=0; ST=0.0D0
      CALL KNOCK(DE,ICOL)
      IF(ICOL.EQ.7) THEN  ! Delta interactions do not count.
        E=E0
        GO TO 110
      ENDIF
      IF(E.GT.DEL(1)) EDP=EDP-E
      GO TO 104
C
  102 CALL START  ! Starts track simulation in current medium.
  103 CONTINUE
      CALL JUMP(DSMAX,DS)  ! Path length to next interaction.
      CALL KNOCK(DE,ICOL)  ! Interaction.
      IF(E.LT.EABS(KPAR,1)) GO TO 104  ! The particle was absorbed.
      GO TO 103
C
  104 CALL SECPAR(LEFT)  ! Any secondary left?
      IF(LEFT.GT.0) THEN
        IF(E.GT.DEL(KPAR)) THEN
          EDP=EDP-E
          GO TO 104
        ELSE
          IF(E.LT.EABS(KPAR,1)) GO TO 104
        ENDIF
        GO TO 102
      ENDIF
C  ------------------------  The simulation of the shower ends here.
C
      EDPT=EDPT+EDP; EDPT2=EDPT2+EDP**2
      CALL TIMER(TSEC)
      IF(TSEC.GT.TDUMP) THEN
        FNT=1.0D0/DBLE(N)
        FG=EDPT*FNT
        EFG=FNT*SQRT(ABS(EDPT2-EDPT**2*FNT))  ! Error (1 sigma level).
        RMEA=SOFTS+FG*SIGTN
        PERR=100.0D0*EFG*SIGTN/RMEA
        WRITE(6,1001) DBLE(N),RMEA,RMEA/STP,SOFTS/STP,PERR,TSEC
        WRITE(26,1001) DBLE(N),RMEA,RMEA/STP,SOFTS/STP,PERR,TSEC
        IF(PERR.LT.TOL.AND.TSEC.GT.TMIN) THEN
          RLEA=RMEA*RHO(1)
          WRITE(27,'(1X,1P,5E13.5,E9.1,E12.4)')
     1      E0,RLEA,RMEA,SOFTS,RMEA/SOFTS,PERR,TSEC
          IF(IETAB.EQ.1) THEN
            NETAB=NETAB+1
            IF(NETAB.GT.NPHD) GO TO 90
            E0=ER(NETAB)
            WRITE(6,'(/,A,1P,E13.6,A)') '#  E0=',E0,' eV'
            WRITE(26,'(/,A,1P,E13.6,A)') '#  E0=',E0,' eV'
            GO TO 30
          ELSE
            GO TO 10
          ENDIF
        ENDIF
        TDUMP=TSEC+DUMP
      ENDIF
 1001 FORMAT(1X,1P,4E13.5,E9.1,E12.4)
      IF(TSEC.GT.TSECA) THEN
        FNT=1.0D0/DBLE(N)
        FG=EDPT*FNT
        EFG=FNT*SQRT(ABS(EDPT2-EDPT**2*FNT))  ! Error (1 sigma level).
        RMEA=SOFTS+FG*SIGTN
        PERR=100.0D0*EFG*SIGTN/RMEA
        WRITE(6,1001) DBLE(N),RMEA,RMEA/STP,SOFTS/STP,PERR,TSEC
        WRITE(26,1001) DBLE(N),RMEA,RMEA/STP,SOFTS/STP,PERR,TSEC
        RLEA=RMEA*RHO(1)
        WRITE(27,'(1X,1P,5E13.5,E9.1,E12.4)')
     1    E0,RLEA,RMEA,SOFTS,RMEA/SOFTS,PERR,TSEC
        IF(IETAB.EQ.1) THEN
          NETAB=NETAB+1
          IF(NETAB.GT.NPHD) GO TO 90
          E0=ER(NETAB)
          WRITE(6,'(/,A,1P,E13.6,A)') '#  E0=',E0,' eV'
          WRITE(26,'(/,A,1P,E13.6,A)') '#  E0=',E0,' eV'
          GO TO 30
        ELSE
          GO TO 10
        ENDIF
      ENDIF
      IF(N.LT.NTOT) GO TO 101
      GO TO 10
C
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C  ***********  Photons.
C
  200 CONTINUE
C  ****  Mass attenuation coefficient.
      RMU0=1.0D0/PRANGE(E0,2,1)
      RMU0R=RMU0/RHO(1)
      WRITE(6,'(A,1P,E13.6,A)') '#  mu/rho=',RMU0R,' cm**2/g'
      WRITE(26,'(A,1P,E13.6,A)') '#  mu/rho=',RMU0R,' cm**2/g'
      WRITE(26,'(A)') '#  F_g = LEA/(E*mu); MEA=F_g*E*mu/rho'
C
      IF(E0.LT.EABS(2,1)) THEN
        PERR=0.0D0
        RMEA=E0*RMU0R
        RLEA=RMEA*RHO(1)
        WRITE(27,'(1X,1P,5E13.5,E9.1,E12.4)')
     1    E0,RLEA,RMEA/E0,RMU0R,RMEA/(E0*RMU0R),PERR,TSEC
        IF(IETAB.EQ.1) THEN
          NETAB=NETAB+1
          IF(NETAB.GT.NPHD) GO TO 90
          E0=ER(NETAB)
          WRITE(6,'(/,A,1P,E13.6,A)') '#  E0=',E0,' eV'
          WRITE(26,'(/,A,1P,E13.6,A)') '#  E0=',E0,' eV'
          GO TO 30
        ELSE
          GO TO 10
        ENDIF
      ELSE
        WRITE(6,2000)
        WRITE(26,2000)
 2000   FORMAT(/,'#',7X,'N',10X,'MEA/E',9X,'F_g',6X,'unc.(%)',
     1    5X,'T(s)')
      ENDIF
      N=0; TDUMP=DUMP
      EDPT=0.0D0; EDPT2=0.0D0
C
C  ------------------------  Shower simulation starts here.
  201 N=N+1
      CALL CLEANS  ! Cleans the secondary stack.
C  ****  Initial state of a primary particle.
      KPAR=2
      E=E0
      EDP=E0
      IF(E.LT.EABS(2,1)) GO TO 204
      X=0.0D0; Y=0.0D0; Z=0.0D0
      U=0.0D0; V=0.0D0; W=1.0D0
      ILB(1)=1; ILB(2)=0; ILB(3)=0; ILB(4)=0; ILB(5)=0
      CALL START
  210 CONTINUE
      CALL JUMP(DSMAX,DS)  ! Path length to next interaction.
      CALL KNOCK(DE,ICOL)
      IF(ICOL.EQ.7) GO TO 210  ! delta scattering.
      IF(E.GT.DEL(2)) EDP=EDP-E
      GO TO 204
C
  202 CALL START  ! Starts track simulation in current medium.
  203 CONTINUE
      CALL JUMP(DSMAX,DS)  ! Path length next interaction.
      CALL KNOCK(DE,ICOL)  ! Interaction.
      IF(E.LT.EABS(KPAR,1)) GO TO 204  ! The particle was absorbed.
      GO TO 203
C
  204 CALL SECPAR(LEFT)  ! Any secondary left?
      IF(LEFT.GT.0) THEN
        IF(E.GT.DEL(KPAR)) THEN
          EDP=EDP-E
          GO TO 204
        ELSE
          IF(E.LT.EABS(KPAR,1)) GO TO 204
        ENDIF
        GO TO 202
      ENDIF
C  ------------------------  The simulation of the shower ends here.
C
      EDPT=EDPT+EDP; EDPT2=EDPT2+EDP**2
      CALL TIMER(TSEC)
      IF(TSEC.GT.TDUMP) THEN
        FNT=1.0D0/DBLE(N)
        FG=EDPT*FNT
        EFG=FNT*SQRT(ABS(EDPT2-EDPT**2*FNT))  ! Error (1 sigma level).
        PERR=100.0D0*EFG/FG
        WRITE(6,2001) DBLE(N),FG*RMU0R/E0,FG/E0,PERR,TSEC
        WRITE(26,2001) DBLE(N),FG*RMU0R/E0,FG/E0,PERR,TSEC
        IF(PERR.LT.TOL.AND.TSEC.GT.TMIN) THEN
          RLEA=FG*RMU0
          RMEA=RLEA/RHO(1)
          WRITE(27,'(1X,1P,5E13.5,E9.1,E12.4)')
     1      E0,RLEA,RMEA/E0,RMU0R,RMEA/(E0*RMU0R),PERR,TSEC
          IF(IETAB.EQ.1) THEN
            NETAB=NETAB+1
            IF(NETAB.GT.NPHD) GO TO 90
            E0=ER(NETAB)
            WRITE(6,'(/,A,1P,E13.6,A)') '#  E0=',E0,' eV'
            WRITE(26,'(/,A,1P,E13.6,A)') '#  E0=',E0,' eV'
            GO TO 30
          ELSE
            GO TO 10
          ENDIF
        ENDIF
        TDUMP=TSEC+DUMP
      ENDIF
 2001 FORMAT(1X,1P,3E13.5,E9.1,E12.4)
      IF(TSEC.GT.TSECA) THEN
        FNT=1.0D0/DBLE(N)
        FG=EDPT*FNT
        EFG=FNT*SQRT(ABS(EDPT2-EDPT**2*FNT))  ! Error (1 sigma level).
        PERR=100.0D0*EFG/FG
        WRITE(6,2001) DBLE(N),FG*RMU0R/E0,FG/E0,PERR,TSEC
        WRITE(26,2001) DBLE(N),FG*RMU0R/E0,FG/E0,PERR,TSEC
        RLEA=FG*RMU0
        RMEA=RLEA/RHO(1)
        WRITE(27,'(1X,1P,5E13.5,E9.1,E12.4)')
     1    E0,RLEA,RMEA/E0,RMU0R,RMEA/(E0*RMU0R),PERR,TSEC
        IF(IETAB.EQ.1) THEN
          NETAB=NETAB+1
          IF(NETAB.GT.NPHD) GO TO 90
          E0=ER(NETAB)
          WRITE(6,'(/,A,1P,E13.6,A)') '#  E0=',E0,' eV'
          WRITE(26,'(/,A,1P,E13.6,A)') '#  E0=',E0,' eV'
          GO TO 30
        ELSE
          GO TO 10
        ENDIF
      ENDIF
      IF(N.LT.NTOT) GO TO 201
      GO TO 10
C
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C  ***********  Positrons.
C
  300 CONTINUE
C  ****  Soft stopping power and total cross section for hard energy-
C        loss interactions.
      CALL PINaT(E0,DEL(3),XH0,XH1,XH2,XS0,XS1,XS2,XT1,XT2,DELTA,1)
      CALL PBRaT(E0,DEL(3),RH0,RH1,RH2,RS1,RS2,1)
      CALL PANaT(E0,TXSAN)
      SIGTN=VMOL(1)*(XH0+RH0+TXSAN)/RHO(1)
      SOFTS=VMOL(1)*(XS1+RS1)/RHO(1)
      STP=VMOL(1)*(XS1+XH1+RS1+RH1)/RHO(1)
      WRITE(6,'(A,1P,E13.6,A)') '#  HardTCS=',SIGTN,' cm**2/g'
      WRITE(6,'(A,1P,E13.6,A)') '#  SoftSTP=',SOFTS,' eV*cm**2/g'
      WRITE(6,'(A,1P,E13.6,A)') '#  STP=',STP,' eV*cm**2/g'
      WRITE(26,'(A,1P,E13.6,A)') '#  HardTCS=',SIGTN,' cm**2/g'
      WRITE(26,'(A,1P,E13.6,A)') '#  SoftSTP=',SOFTS,' eV*cm**2/g'
      WRITE(26,'(A,1P,E13.6,A)') '#  STP=',STP,' eV*cm**2/g'
      WRITE(26,'(A)') '#  LEA/rho in eV*cm**2/gr'
C
      IF(E0.LT.EABS(3,1)) THEN
        PERR=0.0D0
        RMEA=SOFTS
        RLEA=SOFTS*RHO(1)
        WRITE(27,'(1X,1P,5E13.5,E9.1,E12.4)')
     1    E0,RLEA,RMEA,SOFTS,RMEA/SOFTS,PERR,0.0D0
        IF(IETAB.EQ.1) THEN
          NETAB=NETAB+1
          IF(NETAB.GT.NPHD) GO TO 90
          E0=ER(NETAB)
          WRITE(6,'(/,A,1P,E13.6,A)') '#  E0=',E0,' eV'
          WRITE(26,'(/,A,1P,E13.6,A)') '#  E0=',E0,' eV'
          GO TO 30
        ELSE
          GO TO 10
        ENDIF
      ELSE
        WRITE(6,3000)
        WRITE(26,3000)
 3000   FORMAT(/,'#',6X,'N',10X,'LEA/rho',6X,'MEA/STP',5X,'RSTP/STP',
     1    4X,'unc.(%)',5X,'T(s)')
      ENDIF
      N=0; TDUMP=DUMP
      EDPT=0.0D0; EDPT2=0.0D0
C
C  ------------------------  Shower simulation starts here.
  301 N=N+1
      CALL CLEANS  ! Cleans the secondary stack.
C  ****  Initial state of a primary particle.
      KPAR=3
      E=E0
      EDP=E0
      IF(E.LT.EABS(3,1)) GO TO 304
      X=0.0D0; Y=0.0D0; Z=0.0D0
      U=0.0D0; V=0.0D0; W=1.0D0
      ILB(1)=1; ILB(2)=0; ILB(3)=0; ILB(4)=0; ILB(5)=0
      CALL START
  310 CONTINUE
      CALL JUMP(DSMAX,DS)  ! Path length to next interaction.
      MODE=1; P(2)=0.0D0; KSOFTI=0; KDELTA=0; ST=0.0D0
      CALL KNOCK(DE,ICOL)
      IF(ICOL.EQ.7) THEN  ! Delta interactions do not count.
        E=E0
        GO TO 310
      ENDIF
      IF(E.GT.DEL(3)) THEN
        EDP=EDP-E
      ELSE
        EDP=EDP+TREV  ! Positrons carry a latent energy of 1,022 keV.
      ENDIF
      GO TO 304
C
  302 CALL START  ! Starts track simulation in current medium.
  303 CONTINUE
      CALL JUMP(DSMAX,DS)  ! Path length to next interaction.
      CALL KNOCK(DE,ICOL)  ! Interaction.
      IF(E.LT.EABS(KPAR,1)) GO TO 304  ! The particle was absorbed.
      GO TO 303
C
  304 CALL SECPAR(LEFT)  ! Any secondary left?
      IF(LEFT.GT.0) THEN
        IF(E.GT.DEL(KPAR)) THEN
          EDP=EDP-E
          GO TO 304
        ELSE
          IF(E.LT.EABS(KPAR,1)) GO TO 304
        ENDIF
        GO TO 302
      ENDIF
C  ------------------------  The simulation of the shower ends here.
C
      EDPT=EDPT+EDP; EDPT2=EDPT2+EDP**2
      CALL TIMER(TSEC)
      IF(TSEC.GT.TDUMP) THEN
        FNT=1.0D0/DBLE(N)
        FG=EDPT*FNT
        EFG=FNT*SQRT(ABS(EDPT2-EDPT**2*FNT))  ! Error (1 sigma level).
        RMEA=SOFTS+FG*SIGTN
        PERR=100.0D0*EFG*SIGTN/RMEA
        WRITE(6,3001) DBLE(N),RMEA,RMEA/STP,SOFTS/STP,PERR,TSEC
        WRITE(26,3001) DBLE(N),RMEA,RMEA/STP,SOFTS/STP,PERR,TSEC
        IF(PERR.LT.TOL.AND.TSEC.GT.TMIN) THEN
          RLEA=RMEA*RHO(1)
          WRITE(27,'(1X,1P,5E13.5,E9.1,E12.4)')
     1      E0,RLEA,RMEA,SOFTS,RMEA/SOFTS,PERR,TSEC
          IF(IETAB.EQ.1) THEN
            NETAB=NETAB+1
            IF(NETAB.GT.NPHD) GO TO 90
            E0=ER(NETAB)
            WRITE(6,'(/,A,1P,E13.6,A)') '#  E0=',E0,' eV'
            WRITE(26,'(/,A,1P,E13.6,A)') '#  E0=',E0,' eV'
            GO TO 30
          ELSE
            GO TO 10
          ENDIF
        ENDIF
        TDUMP=TSEC+DUMP
      ENDIF
 3001 FORMAT(1X,1P,4E13.5,E9.1,E12.4)
      IF(TSEC.GT.TSECA) THEN
        FNT=1.0D0/DBLE(N)
        FG=EDPT*FNT
        EFG=FNT*SQRT(ABS(EDPT2-EDPT**2*FNT))  ! Error (1 sigma level).
        RMEA=SOFTS+FG*SIGTN
        PERR=100.0D0*EFG*SIGTN/RMEA
        WRITE(6,3001) DBLE(N),RMEA,RMEA/STP,SOFTS/STP,PERR,TSEC
        WRITE(26,3001) DBLE(N),RMEA,RMEA/STP,SOFTS/STP,PERR,TSEC
        RLEA=RMEA*RHO(1)
        WRITE(27,'(1X,1P,5E13.5,E9.1,E12.4)')
     1    E0,RLEA,RMEA,SOFTS,RMEA/SOFTS,PERR,TSEC
        IF(IETAB.EQ.1) THEN
          NETAB=NETAB+1
          IF(NETAB.GT.NPHD) GO TO 90
          E0=ER(NETAB)
          WRITE(6,'(/,A,1P,E13.6,A)') '#  E0=',E0,' eV'
          WRITE(26,'(/,A,1P,E13.6,A)') '#  E0=',E0,' eV'
          GO TO 30
        ELSE
          GO TO 10
        ENDIF
      ENDIF
      IF(N.LT.NTOT) GO TO 301
C
      IF(IETAB.EQ.1) THEN
        NETAB=NETAB+1
        IF(NETAB.GT.NPHD) GO TO 90
        IF(NETAB.LE.NPHD) THEN
          E0=ER(NETAB)
          WRITE(6,'(/,A,1P,E13.6,A)') '#  E0=',E0,' eV'
          WRITE(26,'(/,A,1P,E13.6,A)') '#  E0=',E0,' eV'
          GO TO 30
        ENDIF
      ELSE
        GO TO 10
      ENDIF
C
   90 CONTINUE
      CLOSE(27)
      CLOSE(26)  ! PENELOPE default output unit.
      END
C  *********************************************************************
C                       SUBROUTINE ETABLE
C  *********************************************************************
      SUBROUTINE ETABLE(ER,NPHD,IRES)
C
C     Generates a table of energies with the discontinuities of the
C  photon mass absorption coefficient. IRES is an energy-resolution
C  flag. When IRES=0, the energy grid is minimal; it consists of the
C  energies of the standard grid and the photoabsorption edges.
C
      USE PENELOPE_mod
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER*4 (I-N)
      LOGICAL LREM
C  ****  Composition data.
      COMMON/COMPOS/STF(MAXMAT,30),ZT(MAXMAT),AT(MAXMAT),RHO(MAXMAT),
     1  VMOL(MAXMAT),IZ(MAXMAT,30),NELEM(MAXMAT)
C  ****  Photoelectric cross sections.
      PARAMETER (NTP=12000)
      COMMON/CGPH00/EPH(NTP),XPH(NTP,17),IPHF(99),IPHL(99),NPHS(99),NCUR
C  ****  Standard energy grid.
      PARAMETER (NEGRT=15)
      DIMENSION EGRT(NEGRT)
      DATA EGRT/1.0D0,1.25D0,1.50D0,1.75D0,2.00D0,2.50D0,3.00D0,3.50D0,
     1  4.00D0,5.00D0,6.00D0,7.00D0,8.00D0,9.00D0,1.00D1/
C
      PARAMETER (NDIM=1500)
      DIMENSION ER(NDIM),XSR(NDIM),X1(NDIM),Y1(NDIM),X2(NDIM),Y2(NDIM)
C
      M=1
C
C  ************  Original energy grid for the photoeffect.
C
      IZZ=IZ(M,1)
      IST=IPHF(IZZ)
      LST=IPHL(IZZ)
      NPHD=0
      DO I=IST,LST
        NPHD=NPHD+1
        ER(NPHD)=EXP(EPH(I))
        XSR(NPHD)=1.0D0
      ENDDO
      IF(NELEM(M).GT.1) THEN
        DO IEL=2,NELEM(M)
          N1=NPHD
          DO I=1,N1
            X1(I)=ER(I)
            Y1(I)=XSR(I)
          ENDDO
          IZZ=IZ(M,IEL)
          IST=IPHF(IZZ)
          LST=IPHL(IZZ)
          N2=0
          DO I=IST,LST
            N2=N2+1
            X2(N2)=EXP(EPH(I))
            Y2(N2)=1.0D0
          ENDDO
          CALL MERGE2(X1,Y1,X2,Y2,ER,XSR,N1,N2,NPHD)
        ENDDO
      ENDIF
C  ****  Absorption edges.
      IF(IRES.EQ.0) THEN
        N2=0
        DO I=1,NPHD-1
          IF(ER(I+1)-ER(I).LT.1.0D-14*ER(I)) THEN
            N2=N2+2
            X2(N2+1)=ER(I)
            Y2(N2+1)=1.0D0
            X2(N2+2)=ER(I)
            Y2(N2+2)=1.0D0
            N2=N2+2
          ENDIF
        ENDDO
        GO TO 201
      ENDIF
C
C  ****  Remove grid points with small log-log interpolation errors.
C
      N2=NPHD
      DO J=1,N2
        E=ER(J)
        IF(J.GT.1) THEN
          IF(ABS(E-ER(J-1)).LT.1.0D-7*E) E=E+1.0D-7*E
        ENDIF
        IF(J.LT.N2) THEN
          IF(ABS(ER(J+1)-E).LT.1.0D-7*E) E=E-1.0D-7*E
        ENDIF
        X2(J)=LOG(ER(J))
        CALL GPHaT(E,XS,M)
        XSR(J)=XS
        Y2(J)=LOG(XS)
      ENDDO
C
      EPS=1.0D-4
  102 CONTINUE
      N1=1
      X1(1)=X2(1)
      Y1(1)=Y2(1)
      LREM=.TRUE.
      DO I=2,N2-1
        IF(LREM) THEN
          TST=Y2(I-1)+(Y2(I+1)-Y2(I-1))*(X2(I)-X2(I-1))
     1       /(X2(I+1)-X2(I-1))
          IF(ABS(TST-Y2(I)).GT.EPS*ABS(Y2(I))) THEN
            N1=N1+1
            X1(N1)=X2(I)
            Y1(N1)=Y2(I)
          ELSE
            LREM=.FALSE.
          ENDIF
        ELSE
          N1=N1+1
          X1(N1)=X2(I)
          Y1(N1)=Y2(I)
          LREM=.TRUE.
        ENDIF
      ENDDO
      N1=N1+1
      X1(N1)=X2(N2)
      Y1(N1)=Y2(N2)
      DO I=1,N1
        X2(I)=X1(I)
        Y2(I)=Y1(I)
      ENDDO
C  ****  Iterate.
      IF(N1.LT.N2) THEN
        N2=N1
        GO TO 102
      ENDIF
      N2=N1
      DO I=1,N2
        X2(I)=EXP(X2(I))
      ENDDO
C
C  ************  Standard energy grid.
C
  201 CONTINUE
      N1=0
      IGRID=9
      FGRID=10.0D0
  202 IGRID=IGRID+1
      EV=EGRT(IGRID)*FGRID
      IF(IGRID.EQ.NEGRT) THEN
        IGRID=1
        FGRID=10.0D0*FGRID
      ENDIF
      N1=N1+1
      X1(N1)=EV
      Y1(N1)=1.0D0
      IF(EV.LT.1.0D9) GO TO 202
      CALL MERGE2(X1,Y1,X2,Y2,ER,XSR,N1,N2,NPHD)
C
C  ****  Default energy grid and photoelectric cross sections.
C
      OPEN(7,FILE='Egrid.dat')
      WRITE(7,'(A)') '#     E(eV)      XS_ph (cm**2)'
      DO I=1,NPHD
        E=ER(I)
        IF(I.GT.1) THEN
          IF(ABS(E-ER(I-1)).LT.1.0D-7*E) E=E+1.0D-7*E
        ENDIF
        IF(I.LT.N2) THEN
          IF(ABS(ER(I+1)-E).LT.1.0D-7*E) E=E-1.0D-7*E
        ENDIF
        CALL GPHaT(E,XS,M)
        WRITE(7,'(1P,2E15.7)') ER(I),XS
      ENDDO
      CLOSE(7)
C
      RETURN
      END
