C  ****  Files included to simplify compilation.
      INCLUDE 'penelope.f'
      INCLUDE 'rita.f'
      INCLUDE 'timer.f'
C
C     PROGRAM MUTREN
C
C  This program computes photon mass energy-transfer and mass energy-
C  absorption coefficients from direct simulation by PENELOPE. The cal-
C  culated quantities are f=mu_tr/mu and the product f*(1-g)=mu_en/mu.
C
C                                           Francesc Salvat. July 2017.
C
      USE TRACK_mod
      USE PENELOPE_mod
      USE PENERROR_mod
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER*4 (I-N)
      CHARACTER*20 PMFILE,MFNAME,BUFFER
      DIMENSION PMFILE(MAXMAT)
C  ****  Composition data.
      COMMON/COMPOS/STF(MAXMAT,30),ZT(MAXMAT),AT(MAXMAT),RHO(MAXMAT),
     1  VMOL(MAXMAT),IZ(MAXMAT,30),NELEM(MAXMAT)
C  ****  Energy grid and interpolation constants for the current energy.
      COMMON/CEGRID/EMIN,EL,EU,ET(NEGP),DLEMP(NEGP),DLEMP1,DLFC,
     1  XEL,XE,XEK,KE
C  ****  Electron and positron bremsstrahlung yields.
      COMMON/CBRYLD/EBRY(MAXMAT,NEGP),PBRY(MAXMAT,NEGP)
C
      COMMON/RSEED/ISEED1,ISEED2
C
      PARAMETER (NDIM=1500)
      DIMENSION ER(NDIM)
C
      ISEED1=1; ISEED2=1
      OPEN(26,FILE='mutren-tot.dat')  ! PENELOPE default output unit.
      WRITE(26,'(''#  Photon mass energy-absorption coefficients '',
     1  ''from PENELOPE'')')
C  ****  Material data file.
      WRITE(6,*) '  '
      WRITE(6,*) '  Enter the name of the material data file...'
      READ(5,'(A20)') MFNAME
      WRITE(6,'(''#  Material data file:  '', A24)') MFNAME
      WRITE(26,'(''#  Material data file:  '', A24)') MFNAME
      WRITE(6,*) '  '
      WRITE(6,*) '  To allow using input files with a generic format,'
      WRITE(6,*) '  please, press the ''enter'' key... '
      READ(5,'(A20)') BUFFER
      WRITE(6,*) '  and, press the ''enter'' key again...'
      READ(5,'(A20)') BUFFER
      WRITE(6,*) '  Thank you!'
      WRITE(6,*) '  '
C
C  ****  Simulation parameters and cutoff energies.
C
      NMAT=1
      PMFILE(1)=MFNAME
      EABS(1,1)=50.0D0; EABS(2,1)=50.0D0; EABS(3,1)=50.0D0
      C1(1)=0.2D0; C2(1)=0.2D0
      WCC(1)=1.0D3; WCR(1)=1.0D3
      EPMAX=1.0D9
      EABS1=EABS(1,1)
C
C  ****  Initialisation.
C
      WRITE(6,*) '  Initialising PENELOPE ...'
      IWR=12
      OPEN(IWR,FILE='material.dat')
      INFO=3
      CALL PEINIT(EPMAX,NMAT,IWR,INFO,PMFILE)
      IF(IRETRN.NE.0) STOP
      CLOSE(IWR)
      CALL ETABLE(ER,NPHD,1)
C
      F=1.0D0  ! To prevent compilation warnings.
      FG=1.0D0
C
C  ****  Tolerance. The simulation stops when the statistical
C  uncertainty of the calculated LEA value (1-sigma) is less than TOL.
C
      WRITE(6,*) '  '
      WRITE(6,*) '  Tolerance?  (in %, typically 0.1)'
      READ(5,*) TOL
      IF(TOL.GT.1.0D0) TOL=1.0D0
      WRITE(6,'(A,1P,E13.6,A)') '#  Tolerance =',TOL,' %'
      WRITE(26,'(A,1P,E13.6,A)') '#  Tolerance =',TOL,' %'
C
C  ****  Allotted simulation time (in seconds), for each energy.
C
      WRITE(6,*) '  '
      WRITE(6,*) '  Allotted simulation time (in seconds)?'
      READ(5,*) TSECA
      IF(TSECA.LT.0.0D0) TSECA=900.0D0
      WRITE(6,'(A,1P,E13.6,A)') '#  Allotted simulation time =',
     1   TSECA,' s'
      WRITE(26,'(A,1P,E13.6,A)') '#  Allotted simulation time =',
     1   TSECA,' s'
C
      DUMP=10.0D0
      WRITE(26,'(A,1P,E13.6,A)') '#  Dump period=',DUMP,' s'
      WRITE(26,'(A)') '#  mu_en/rho in g/cm**2'
C
C  ****  Output file with tables of mass-energy absorption coefficients.
C
      OPEN(27,FILE='mutren.dat')
      WRITE(27,'(''#  Photon mass energy-absorption coefficients from'',
     1 '' PENELOPE'')')
      WRITE(27,'(''#  Material data file:  '', A24)') MFNAME
      WRITE(27,'(A,1P,E13.6,A)') '#  Mass density=',RHO(1),' g/cm**3'
      WRITE(27,'(A)') '# '
      WRITE(27,'(A,1P,E13.6,A)') '#  Tolerance =',TOL,' %'
      WRITE(27,'(A,1P,E13.6,A)') '#  Allotted simulation time =',
     1   TSECA,' s'
      WRITE(27,'(A,1P,E13.6,A)') '#  Dump period=',DUMP,' s'
      WRITE(27,1000)
 1000 FORMAT('#',/,'#',4X,'Energy',7X,'mu/rho',6x,'mu_en/rho',5X,
     1  '(1-g)*f',8X,'f',12X,'1-g',6X,'uncert.',6X,'T',/,'#',5X,
     1  '(eV)',7X,'(cm**2/g)',4X,'(cm**2/g)',44X,'(%)',7X,'(s)',/,
     1  '# ',99('-'))
C
C  ****  Photon energy.
C
      IETAB=0
      NETAB=1
      WRITE(6,*) '  '
      WRITE(6,'(3X,2A)') 'TRICK: to use the default energy grid, enter',
     1  ' a value of the'
      WRITE(6,'(10X,2A)') 'photon energy, E0, less than zero. The ',
     1  'calculation will'
      WRITE(6,'(10X,2A)') 'start at an energy greater than or equal',
     1  ' to ABS(E0).'
 10   CONTINUE
      WRITE(6,*) '  '
      WRITE(6,*) '  Photon energy (in eV)?'
      READ(5,*,ERR=90,END=90) E0
      IF(E0.LT.0.0D0) THEN
        IETAB=1
        ESTART=MAX(ABS(E0)-10.0D0,1.0D2)
 20     CONTINUE
        NETAB=NETAB+1
        IF(NETAB.GT.NPHD) GO TO 90
        E0=ER(NETAB)
        IF(E0.LT.ESTART) GO TO 20
      ENDIF
      WRITE(6,'(/,A,1P,E13.6,A)') '#  E0=',E0,' eV'
      WRITE(26,'(/,A,1P,E13.6,A)') '#  E0=',E0,' eV'
C
 30   CONTINUE
      IF(IETAB.EQ.1) THEN
        IF(NETAB.GT.1) THEN
          IF(ABS(E0-ER(NETAB-1)).LT.1.0D-12*E0) E0=E0+1.0D-5
        ENDIF
        IF(NETAB.LT.NPHD) THEN
          IF(ABS(E0-ER(NETAB+1)).LT.1.0D-12*E0) E0=E0-1.0D-5
        ENDIF
      ENDIF
C
      DO I=NEGP,1,-1
        IF(EXP(EBRY(1,I))*ET(I).LT.1.0D-4*E0) THEN
          EABS(1,1)=MAX(ET(I),EABS1)
          GO TO 40
        ENDIF
      ENDDO
  40  CONTINUE
C
      EABS(3,1)=MIN(MAX(1.0D-4*E0,5.0D3),EABS(1,1))
C
      WRITE(6,'(A,1P,E13.6,A)') '#  EABS(el)=',EABS(1,1),' eV'
      WRITE(6,'(A,1P,E13.6,A)') '#  EABS(ph)=',EABS(2,1),' eV'
      WRITE(6,'(A,1P,E13.6,A)') '#  EABS(po)=',EABS(3,1),' eV'
      WRITE(26,'(A,1P,E13.6,A)') '#  EABS(el)=',EABS(1,1),' eV'
      WRITE(26,'(A,1P,E13.6,A)') '#  EABS(ph)=',EABS(2,1),' eV'
      WRITE(26,'(A,1P,E13.6,A)') '#  EABS(po)=',EABS(3,1),' eV'
      DSMAX=1.0D20
C
      NTOT=1000000000
      WGHT=1.0D0
      CALL TIME0
      MAT=1
      IBODY=1
      TMIN=DUMP
C
      RMU0=1.0D0/PRANGE(E0,2,1)
      RMU0=RMU0/RHO(1)
      WRITE(6,'(A,1P,E13.6,A)') '#  mu/rho=',RMU0,' cm**2/g'
      WRITE(26,'(A,1P,E13.6,A)') '#  mu/rho=',RMU0,' cm**2/g'
C
      WRITE(6,2000)
      WRITE(26,2000)
 2000 FORMAT(/,'#',7X,'N',8X,'mu_en/rho',5X,'(1-g)*f',9X,'f',7X,
     1  'unc.(%)',5X,'T(s)')
      N=0; TDUMP=DUMP
      EDPT=0.0D0; EDPT2=0.0D0
      ETRT=0.0D0; ETRT2=0.0D0
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C  ------------------------  Shower simulation starts here.
  201 N=N+1
      CALL CLEANS  ! Cleans the secondary stack.
C  ****  Initial state of a primary particle.
      KPAR=2
      E=E0
      X=0.0D0; Y=0.0D0; Z=0.0D0
      U=0.0D0; V=0.0D0; W=1.0D0
      ILB(1)=1; ILB(2)=0; ILB(3)=0; ILB(4)=0; ILB(5)=0
      CALL START
  210 CONTINUE
      CALL JUMP(DSMAX,DS)  ! Path length next interaction.
      CALL KNOCK(DE,ICOL)
      IF(ICOL.EQ.7) GO TO 210  ! delta scattering.
      EDP=E0-E
      ETR=EDP
      GO TO 204
C
  202 CALL START  ! Starts track simulation in current medium.
  203 CONTINUE
      CALL JUMP(DSMAX,DS)  ! Path length next interaction.
      CALL KNOCK(DE,ICOL)  ! Interaction.
      IF(E.LT.EABS(KPAR,1)) GO TO 204  ! The particle was absorbed.
      GO TO 203
C
  204 CALL SECPAR(LEFT)  ! Any secondary left?
      IF(LEFT.GT.0) THEN
        IF(KPAR.EQ.2) THEN
          EDP=EDP-E
          IF(ILB(1).EQ.2) ETR=ETR-E
          GO TO 204
        ELSE
          IF(E.LT.EABS(KPAR,1)) GO TO 204
        ENDIF
        GO TO 202
      ENDIF
C  ------------------------  The simulation of the shower ends here.
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
      EDPT=EDPT+EDP; EDPT2=EDPT2+EDP**2
      ETRT=ETRT+ETR; ETRT2=ETRT2+ETR**2
      CALL TIMER(TSEC)
      IF(TSEC.GT.TDUMP) THEN
        FNT=1.0D0/DBLE(N)
        F=ETRT*FNT
        FG=EDPT*FNT
        EFG=FNT*SQRT(ABS(EDPT2-EDPT**2*FNT))  ! Error (1 sigma level).
        PERR=100.0D0*EFG/FG
        WRITE(6,2001) DBLE(N),FG*RMU0/E0,FG/E0,F/E0,PERR,TSEC
        WRITE(26,2001) DBLE(N),FG*RMU0/E0,FG/E0,F/E0,PERR,TSEC
        IF(PERR.LT.TOL.AND.TSEC.GT.TMIN) THEN
          WRITE(27,'(1X,1P,6E13.5,E9.1,E12.4)')
     1      E0,RMU0,FG*RMU0/E0,FG/E0,F/E0,FG/F,PERR,TSEC
          IF(IETAB.EQ.1) THEN
            NETAB=NETAB+1
            IF(NETAB.GT.NPHD) GO TO 90
            IF(NETAB.LE.NPHD) THEN
              E0=ER(NETAB)
              WRITE(6,'(/,A,1P,E13.6,A)') '#  E0=',E0,' eV'
              WRITE(26,'(/,A,1P,E13.6,A)') '#  E0=',E0,' eV'
              GO TO 30
            ENDIF
          ELSE
            GO TO 10
          ENDIF
        ENDIF
        TDUMP=TSEC+DUMP
      ENDIF
 2001 FORMAT(1X,1P,4E13.5,E9.1,E12.4)
      IF(TSEC.GT.TSECA) THEN
        FNT=1.0D0/DBLE(N)
        F=ETRT*FNT
        FG=EDPT*FNT
        EFG=FNT*SQRT(ABS(EDPT2-EDPT**2*FNT))  ! Error (1 sigma level).
        PERR=100.0D0*EFG/FG
        WRITE(6,2001) DBLE(N),FG*RMU0/E0,FG/E0,F/E0,PERR,TSEC
        WRITE(26,2001) DBLE(N),FG*RMU0/E0,FG/E0,F/E0,PERR,TSEC
        WRITE(27,'(1X,1P,6E13.5,E9.1,E12.4)')
     1    E0,RMU0,FG*RMU0/E0,FG/E0,F/E0,FG/F,PERR,TSEC
        IF(IETAB.EQ.1) THEN
          NETAB=NETAB+1
          IF(NETAB.GT.NPHD) GO TO 90
          IF(NETAB.LE.NPHD) THEN
            E0=ER(NETAB)
            WRITE(6,'(/,A,1P,E13.6,A)') '#  E0=',E0,' eV'
            WRITE(26,'(/,A,1P,E13.6,A)') '#  E0=',E0,' eV'
            GO TO 30
          ENDIF
        ELSE
          GO TO 10
        ENDIF
      ENDIF
      IF(N.LT.NTOT) GO TO 201
C
      IF(IETAB.EQ.1) THEN
        NETAB=NETAB+1
        IF(NETAB.GT.NPHD) GO TO 90
        IF(NETAB.LE.NPHD) THEN
          E0=ER(NETAB)
          WRITE(6,'(/,A,1P,E13.6,A)') '#  E0=',E0,' eV'
          WRITE(26,'(/,A,1P,E13.6,A)') '#  E0=',E0,' eV'
          GO TO 30
        ENDIF
      ELSE
        GO TO 10
      ENDIF
C
   90 CONTINUE
      CLOSE(27)
      CLOSE(26)  ! PENELOPE default output unit.
      END
C  *********************************************************************
C                       SUBROUTINE ETABLE
C  *********************************************************************
      SUBROUTINE ETABLE(ER,NPHD,IRES)
C
C     Generates a table of energies with the discontinuities of the
C  photon mass absorption coefficient. IRES is an energy-resolution
C  flag. When IRES=0, the energy grid is minimal; it consists of the
C  energies of the standard grid and the photoabsorption edges.
C
      USE PENELOPE_mod
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER*4 (I-N)
      LOGICAL LREM
C  ****  Composition data.
      COMMON/COMPOS/STF(MAXMAT,30),ZT(MAXMAT),AT(MAXMAT),RHO(MAXMAT),
     1  VMOL(MAXMAT),IZ(MAXMAT,30),NELEM(MAXMAT)
C  ****  Photoelectric cross sections.
      PARAMETER (NTP=12000)
      COMMON/CGPH00/EPH(NTP),XPH(NTP,17),IPHF(99),IPHL(99),NPHS(99),NCUR
C  ****  Standard energy grid.
      PARAMETER (NEGRT=15)
      DIMENSION EGRT(NEGRT)
      DATA EGRT/1.0D0,1.25D0,1.50D0,1.75D0,2.00D0,2.50D0,3.00D0,3.50D0,
     1  4.00D0,5.00D0,6.00D0,7.00D0,8.00D0,9.00D0,1.00D1/
C
      PARAMETER (NDIM=12000)
      DIMENSION ER(NDIM),XSR(NDIM),X1(NDIM),Y1(NDIM),X2(NDIM),Y2(NDIM)
C
      M=1
C
C  ************  Original energy grid for the photoeffect.
C
      IZZ=IZ(M,1)
      IST=IPHF(IZZ)
      LST=IPHL(IZZ)
      NPHD=0
      DO I=IST,LST
        NPHD=NPHD+1
        ER(NPHD)=EXP(EPH(I))
        XSR(NPHD)=1.0D0
      ENDDO
      IF(NELEM(M).GT.1) THEN
        DO IEL=2,NELEM(M)
          N1=NPHD
          DO I=1,N1
            X1(I)=ER(I)
            Y1(I)=XSR(I)
          ENDDO
          IZZ=IZ(M,IEL)
          IST=IPHF(IZZ)
          LST=IPHL(IZZ)
          N2=0
          DO I=IST,LST
            N2=N2+1
            X2(N2)=EXP(EPH(I))
            Y2(N2)=1.0D0
          ENDDO
          CALL MERGE2(X1,Y1,X2,Y2,ER,XSR,N1,N2,NPHD)
        ENDDO
      ENDIF
C  ****  Absorption edges.
      IF(IRES.EQ.0) THEN
        N2=0
        DO I=1,NPHD-1
          IF(ER(I+1)-ER(I).LT.1.0D-14*ER(I)) THEN
            N2=N2+2
            X2(N2+1)=ER(I)
            Y2(N2+1)=1.0D0
            X2(N2+2)=ER(I)
            Y2(N2+2)=1.0D0
            N2=N2+2
          ENDIF
        ENDDO
        GO TO 201
      ENDIF
C
C  ****  Remove grid points with small log-log interpolation errors.
C
      N2=NPHD
      DO J=1,N2
        E=ER(J)
        IF(J.GT.1) THEN
          IF(ABS(E-ER(J-1)).LT.1.0D-7*E) E=E+1.0D-7*E
        ENDIF
        IF(J.LT.N2) THEN
          IF(ABS(ER(J+1)-E).LT.1.0D-7*E) E=E-1.0D-7*E
        ENDIF
        X2(J)=LOG(ER(J))
        CALL GPHaT(E,XS,M)
        XSR(J)=XS
        Y2(J)=LOG(XS)
      ENDDO
C
      EPS=1.0D-4
  102 CONTINUE
      N1=1
      X1(1)=X2(1)
      Y1(1)=Y2(1)
      LREM=.TRUE.
      DO I=2,N2-1
        IF(LREM) THEN
          TST=Y2(I-1)+(Y2(I+1)-Y2(I-1))*(X2(I)-X2(I-1))
     1       /(X2(I+1)-X2(I-1))
          IF(ABS(TST-Y2(I)).GT.EPS*ABS(Y2(I))) THEN
            N1=N1+1
            X1(N1)=X2(I)
            Y1(N1)=Y2(I)
          ELSE
            LREM=.FALSE.
          ENDIF
        ELSE
          N1=N1+1
          X1(N1)=X2(I)
          Y1(N1)=Y2(I)
          LREM=.TRUE.
        ENDIF
      ENDDO
      N1=N1+1
      X1(N1)=X2(N2)
      Y1(N1)=Y2(N2)
      DO I=1,N1
        X2(I)=X1(I)
        Y2(I)=Y1(I)
      ENDDO
C  ****  Iterate.
      IF(N1.LT.N2) THEN
        N2=N1
        GO TO 102
      ENDIF
      N2=N1
      DO I=1,N2
        X2(I)=EXP(X2(I))
      ENDDO
C
C  ************  Standard energy grid.
C
  201 CONTINUE
      N1=0
      IGRID=9
      FGRID=10.0D0
  202 IGRID=IGRID+1
      EV=EGRT(IGRID)*FGRID
      IF(IGRID.EQ.NEGRT) THEN
        IGRID=1
        FGRID=10.0D0*FGRID
      ENDIF
      N1=N1+1
      X1(N1)=EV
      Y1(N1)=1.0D0
      IF(EV.LT.1.0D9) GO TO 202
      CALL MERGE2(X1,Y1,X2,Y2,ER,XSR,N1,N2,NPHD)
C
C  ****  Default energy grid and photoelectric cross sections.
C
      OPEN(7,FILE='Egrid.dat')
      WRITE(7,'(A)') '#     E(eV)      XS_ph (cm**2)'
      DO I=1,NPHD
        E=ER(I)
        IF(I.GT.1) THEN
          IF(ABS(E-ER(I-1)).LT.1.0D-7*E) E=E+1.0D-7*E
        ENDIF
        IF(I.LT.N2) THEN
          IF(ABS(ER(I+1)-E).LT.1.0D-7*E) E=E-1.0D-7*E
        ENDIF
        CALL GPHaT(E,XS,M)
        WRITE(7,'(1P,2E15.7)') ER(I),XS
      ENDDO
      CLOSE(7)
C
      RETURN
      END
