# Gnuplot version 5.0 patchlevel 3.
# Plots interaction data tables generated by tables.f

#-----------------------------------------------------------------------

reset

# Define the appropriate terminal
 set terminal wxt size 900,600 enhanced font 'Arial,12' dashlength 2.0
#set terminal windows size 900,600 enhanced font 'Arial,12'
#set terminal qt size 900,600 font 'Arial,12' enhanced

set encoding utf8  # Character encoding
set mouse
set zero 1.0e-99

# Soft colors
#set linetype  1 lw 2 lc rgb 29390     # UB blue
#set linetype  2 lw 2 lc rgb 13382400  # soft red
#set linetype  3 lw 2 lc rgb 39168     # dark green
#set linetype  4 lw 2 lc rgb 10053171  # brown
#set linetype  5 lw 2 lc rgb 16724940  # pink

set linetype  1 lw 2 lc rgb 255       # blue
set linetype  2 lw 2 lc rgb 16711680  # red
set linetype  3 lw 2 lc rgb 65280     # green
set linetype  4 lw 2 lc rgb 10053171  # brown
set linetype  5 lw 2 lc rgb 16711935  # magenta

set linetype 10 lw 2 lc rgb 0         # black
set linetype 11 lw 2 lc rgb 8421504   # 50 percent black
set xzeroaxis linestyle 11
set yzeroaxis linestyle 11

#set palette rgbformulae 7,5,15
set palette defined ( 1 "blue", 2 "green", 3 "yellow" , 4 "orange", 5 "red")
#set palette defined ( 1 "blue", 2 "green", 3 "yellow" , 4 "white")
#set palette defined ( 1 "red", 2 "orange", 3 "yellow" , 4 "white")
set linetype 100 lw 1 lc rgb 0         # black

set dashtype 1 (8,3)       # dashed
set dashtype 2 (12,3,1,3)  # dot-dashed
set dashtype 3 (4,3)       # dotted

set tmargin at screen 0.90
set bmargin at screen 0.13
set rmargin at screen 0.94
set lmargin at screen 0.15

set size 1,1  # The plot fills the available canvas
set border lw 2.0
set pointsize 0.8
set bars 0.5

#set format xy "10^{%T}"
#set format xy "%.1te%+-3T"
#set format cb "%.1te%+-3T"

#-----------------------------------------------------------------------

#! tables

set zero 1.0e-35
set format xy "%.1tE%+-3T"

set mxtics 10
set mytics 10
set grid xtics ytics mxtics mytics ls 11 lw 1, ls 11 lw 1
unset grid

set title 'Photon interaction cross sections'
set key on right top Right samplen 5.5 height +1 width +1
set xlabel 'Energy   (eV)'
set ylabel 'Cross section   (cm^2)'
set xrange [50.0:1.0e9]
set yrange [1.0E-30:*]
set logscale x
set logscale y
plot 'MACgRA.tab' u 1:2 title 'Rayleigh' w l ls 1, \
     'TCSg.tab' u 1:3 title 'Compton' w l ls 3, \
     'TCSg.tab' u 1:4 title 'Photoabsorption' w l ls 2, \
     'TCSg.tab' u 1:5 title 'Pair production' w l ls 4, \
     'TCSg.tab' u 1:6 title 'Triplet production' w l ls 5, \
     'TCSg.tab' u 1:7 title 'Total' w l ls 10
pause -1

set title 'Electron interaction cross sections'
set key on right bottom Right samplen 5.5 height +1 width +1
set xlabel 'Energy   (eV)'
set ylabel 'Cross section   (cm^2)'
set yrange [1.0e-28:*]
set xrange [50.0:1.0e9]
set logscale x
set logscale y
plot 'TCSe.tab' u 1:2 title 'Elastic' w l ls 1, \
     'TCSe.tab' u 1:3 title 'Inelastic' w l ls 3, \
     'TCSe.tab' u 1:4 title 'Bremsstrahlung' w l ls 2, \
     'TCSe.tab' u 1:6 title 'Inner shell ion.' w l ls 4, \
     'TCSe.tab' u 1:5 title 'Total' w l ls 10
pause -1

set title 'Positron interaction cross sections'
set key on right bottom Right samplen 5.5 height +1 width +1
set xlabel 'Energy   (eV)'
set ylabel 'Cross section   (cm^2)'
set yrange [1.0e-28:*]
set xrange [50.0:1.0e9]
set logscale x
set logscale y
plot 'TCSp.tab' u 1:2 title 'Elastic' w l ls 1, \
     'TCSp.tab' u 1:3 title 'Inelastic' w l ls 3, \
     'TCSp.tab' u 1:4 title 'Bremsstrahlung' w l ls 2, \
     'TCSp.tab' u 1:7 title 'Inner shell ion.' w l ls 4, \
     'TCSp.tab' u 1:5 title 'Annihilation' w l ls 5, \
     'TCSp.tab' u 1:6 title 'Total' w l ls 10
pause -1
