C  Minimal MAIN PROGRAM for tracking particles in quadric geometries
C  with electromagnetic fields. The commented blocks with the string
C  'EM field' indicate modifications needed to describe the effect of
C  EM forces.
C
C  ****  Files included to simplify compilation.
      INCLUDE 'penelope.f'
      INCLUDE 'rita.f'
      INCLUDE 'pengeom.f'
      INCLUDE 'penfield.f'    ! EM field.
      INCLUDE 'timer.f'
C
      USE TRACK_mod
      USE PENELOPE_mod
      USE PENERROR_mod
      USE PENGEOM_mod
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER*4 (I-N)
C
      CHARACTER*20 PMFILE
      DIMENSION PMFILE(MAXMAT)
      DIMENSION PARINP(20)
      DIMENSION DSMAX(NB)
C
      COMMON/RSEED/ISEED1,ISEED2
C

C>>>>>>>>> EM field >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
C  ****  EM field.
      COMMON/UFIELD/EFX,EFY,EFZ,BFX,BFY,BFZ
C<<<<<<<<< EM field >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

      ISEED1=1; ISEED2=1
C
      KPARP=1  ! Primary particles: 1=electron, 2=photon, 3=positron.
      WRITE(6,'(3X,A,I3)') 'KPARP=',KPARP
      E0=6.0E6  ! Initial energy.
      WRITE(6,'(3X,A,1P,E13.6)') 'E0=',E0
C
      WRITE(6,*) '  Initialising PENELOPE ...'
      NMAT=1
      PMFILE(1)='H2O.mat'
      DO M=1,NMAT
        EABS(1,M)=1.0D5; EABS(2,M)=1.0D4; EABS(3,M)=1.0D5
        C1(M)=0.01D0; C2(M)=0.01D0
        WCC(M)=5.0D3; WCR(M)=5.0D3
      ENDDO
      EPMAX=E0
C
      OPEN(26,FILE='pm-field.dat')  ! PENELOPE default output unit.
      OPEN(IWR,FILE='material.dat')
      INFO=1
      CALL PEINIT(EPMAX,NMAT,IWR,INFO,PMFILE)
      IF(IRETRN.NE.0) GO TO 105
      CLOSE(IWR)
C
      WRITE(6,*) '  Initialising PENGEOM ...'
      OPEN(15,FILE='pm-field.geo')
      OPEN(16,FILE='geometry.rep')
      NPINP=0
      CALL GEOMIN(PARINP,NPINP,NMATG,NBODY,15,16)
      IF(IRETRN.NE.0) GO TO 105
      CLOSE(15)
      CLOSE(16)
      DSMAX(1)=1.0D20

C>>>>>>>>> EM field >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
C  ****  Initialisation of the electromagnetic-field subroutines.
      OPEN(29,FILE='track')  ! Output file with generated tracks.
      EFX=0.0D0; EFY=0.0D0 ;EFZ=0.0D0
      WRITE(26,'(/3X,''E-field (V/cm)  ='',1P,3E15.7)') EFX,EFY,EFZ
      WRITE(29,'('' # E-field (V/cm)  ='',1P,3E15.7)') EFX,EFY,EFZ
      BFX=1.5D4; BFY=0.0D0; BFZ=0.0D0
      WRITE(26,'(/3X,''B-field (gauss) ='',1P,3E15.7)') BFX,BFY,BFZ
      WRITE(29,'('' # B-field (gauss) ='',1P,3E15.7)') BFX,BFY,BFZ
C
      IF(KPARP.EQ.1) THEN
        WRITE(29,*) '# Electron'
      ELSE IF(KPARP.EQ.3) THEN
        WRITE(29,*) '# Positron'
      ELSE
        STOP 'Please, give me a charged particle....'
      ENDIF
      ULDV=1.0D-2  ! Maximum fractional changes in direction,
      ULDE=1.0D-2  ! energy,
      ULEM=1.0D-2  ! and fields.
C<<<<<<<<< EM field >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

      WGHT=1.0D0
      NTOT=15
      CALL TIME0
      TSECA=3600.0D0
      N=0
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C  ------------------------  Shower simulation starts here.
  101 N=N+1
      CALL CLEANS  ! Cleans the secondary stack.
C  ****  Initial state of a primary particle.
      KPAR=KPARP
      E=E0  ! Must be larger than EABS.
      X=0.0D0; Y=0.0D0; Z=0.0D0
      U=0.0D0; V=0.0D0; W=1.0D0
      ILB(1)=1; ILB(2)=0; ILB(3)=0; ILB(4)=0; ILB(5)=0
      CALL LOCATE
      IF(MAT.EQ.0) THEN  ! The particle starts from vacuum.
        CALL STEP(1.0D30,DSEF,NCROSS)
        IF(MAT.EQ.0) GO TO 104  ! The particle left the system.
      ENDIF
C
  102 CALL START  ! Starts track simulation in current medium.
      IF(IRETRN.NE.0) GO TO 105

C>>>>>>>>> EM field >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
      WRITE(29,*) '  '  ! Starts the plot of a new track.
      IF(KPAR.NE.2) write(29,'(1x,1p,4e15.7,i3)') x,y,z,e,kpar
C<<<<<<<<< EM field >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

C
  103 CONTINUE
      MATL=MAT
*     CALL JUMP(DSMAX(IBODY),DS)  ! Path length next interaction.
*     CALL STEP(DS,DSEF,NCROSS)  ! Determines the step end position.

C>>>>>>>>> EM field >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
      CALL TPEMF0(ULDV,ULDE,ULEM,TMAX)
      IF(IRETRN.NE.0) GO TO 105
      DSMAXL=MIN(TMAX,DSMAX(IBODY))
      CALL JUMP(DSMAXL,DS)
      CALL TPEMF1(DS,DSEF,NCROSS)
      IF(KPAR.NE.2) THEN ! Soft energy loss
        IF(DESOFT.GT.0.0D0) DE=SSOFT*DSEF
      ENDIF
      IF(IRETRN.NE.0) GO TO 105
      IF(E.LT.EABS(KPAR,MATL)) GO TO 104
      IF(KPAR.NE.2) write(29,'(1x,1p,4e15.7,i3)') x,y,z,e,kpar
C<<<<<<<<< EM field >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

      IF(NCROSS.GT.0) THEN
        IF(KPAR.NE.2) E=E0SEGM-SSOFT*DSEF ! Soft energy loss correction
        IF(MAT.EQ.0) GO TO 104
        GO TO 102
      ENDIF
C
      CALL KNOCK(DE,ICOL) ! Simulates the interaction event
      IF(IRETRN.NE.0) GO TO 105
      IF(KPAR.NE.2) THEN ! Ignore energy loss in the hinge
        IF(ICOL.EQ.1) DE=0.0D0
      ENDIF
C
      IF(E.LT.EABS(KPAR,MAT)) GO TO 104  ! The particle was absorbed.
      GO TO 103
C
  104 CALL SECPAR(LEFT)  ! Any secondary left?
      IF(LEFT.GT.0) GO TO 102
C  ------------------------  The simulation of the shower ends here.
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
      CALL TIMER(TSEC)
      IF(TSEC.GT.TSECA) STOP
      IF(N.LT.NTOT) GO TO 101
      CLOSE(26)  ! PENELOPE default output unit.
      CLOSE(29)  ! Local unit (file with generated tracks).
C
 105  CONTINUE
      IF(IRETRN.NE.0) THEN
        WRITE(6,'(A)') REASON
        WRITE(6,'(A,I5)') 'IRETRN =',IRETRN
      ENDIF
C
      END
