C  ****  Files included to simplify compilation.
      INCLUDE 'penelope.f'
      INCLUDE 'rita.f'

C  This is a modification of the program TABLES.F, suggested by F. Tola,
C  that displays the mean-free paths for hard events of electrons and
C  positrons. This information is useful to analyze the effect of the
C  simulation parameters on the speed of mixed (class-II) simulations
C  of electrons and positrons.

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                      C
C  TTTTTT    AA    BBBBB   L       EEEEEE   SSSS          IIII  IIII   C
C    TT     A  A   B    B  L       E       S    S          II    II    C
C    TT    A    A  B    B  L       E       SS      XXXXX   II    II    C
C    TT    AAAAAA  BBBBB   L       EEEE      SSSS  XXXXX   II    II    C
C    TT    A    A  B    B  L       E       S    S          II    II    C
C    TT    A    A  BBBBB   LLLLLL  EEEEEE   SSSS          IITI  IITI   C
C                                                                      C
C                                                   (version 2018).    C
C                                                                      C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                      C
C  PENELOPE/PENGEOM (version 2018)                                     C
C  Copyright (c) 2001-2018                                             C
C  Universitat de Barcelona                                            C
C                                                                      C
C  Permission to use, copy, modify, distribute and sell this software  C
C  and its documentation for any purpose is hereby granted without     C
C  fee, provided that the above copyright notice appears in all        C
C  copies and that both that copyright notice and this permission      C
C  notice appear in all supporting documentation. The Universitat de   C
C  Barcelona makes no representations about the suitability of this    C
C  software for any purpose. It is provided 'as is' without express    C
C  or implied warranty.                                                C
C                                                                      C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C  TABLES-CLASS-II reads material properties and cross sections from a
C  material-data file, which was previously generated by running the
C  program MATERIAL.
C
C  ****  Code number (ICOL) for the various interaction events:
C
C     +----+-----------------+-----------------+-----------------+
C     |ICOL|electron (KPAR=1)|photon (KPAR=2)  |positron (KPAR=3)|
C     +----+-----------------+-----------------+-----------------+
C     | 1  |hinge            |Rayleigh         |hinge            |
C     +----+-----------------+-----------------+-----------------+
C     | 2  |elastic          |Compton          |elastic          |
C     +----+-----------------+-----------------+-----------------+
C     | 3  |inelastic        |photoabsorption  |inelastic        |
C     +----+-----------------+-----------------+-----------------+
C     | 4  |bremsstrahlung   |pair production  |bremsstrahlung   |
C     +----+-----------------+-----------------+-----------------+
C     | 5  |inner-shell ion. |not defined      |inner-shell ion. |
C     +----+-----------------+-----------------+-----------------+
C     | 6  |not defined      |not defined      |annihilation     |
C     +----+-----------------+-----------------+-----------------+
C     | 7  |delta scattering |delta scattering |delta scattering |
C     +----+-----------------+-----------------+-----------------+
C     | 8  |not defined      |not defined      |not defined      |
C     +----+-----------------+-----------------+-----------------+
C
C
C  *********************************************************************
C                       MAIN PROGRAM
C  *********************************************************************
      USE PENELOPE_mod
      USE PENERROR_mod
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER*4 (I-N)
      CHARACTER MFNAME*20,PMFILE*20
      DIMENSION PMFILE(MAXMAT)
C
C  ****  Material data file.
C
      WRITE(6,*) '  '
      WRITE(6,*) 'Enter the name of the material data file...'
      READ(5,'(A20)') MFNAME
      WRITE(6,'('' Material data file:  '', A24)') MFNAME
C
      OPEN(9,FILE=MFNAME,STATUS='OLD',IOSTAT=KODE)
      IF(KODE.NE.0) THEN
        WRITE(6,*) '  '
        WRITE(6,'('' *** This material file does not exist.'')')
        STOP
      ENDIF
      CLOSE(9)
C
      EMAX=1.0D9
C
      WRITE(6,*) '  '
      WRITE(6,*) 'Simulation parameters for electrons/positrons.'
      WRITE(6,*) 'Please, enter C1, C2, WCC (eV) and WCR (eV)...'
      READ(5,*,ERR=10) C11,C21,WCC1,WCR1
      C1(1)=C11
      C2(1)=C21
      WCC(1)=WCC1
      WCR(1)=WCR1
   10 CONTINUE
      IF(WCR(1).LT.10.0D0) WCR(1)=-10.0D0
      WRITE(6,1000) C1(1),C2(1),WCC(1),WCR(1)
 1000 FORMAT(6X,'C1 =',1P,E11.4,',       C2 =',E11.4,/5X,'WCC =',
     2    E11.4,' eV,   WCR =',E11.4,' eV',/)
      EABS(1,1)=50.0D0
      EABS(2,1)=50.0D0
      EABS(3,1)=50.0D0
C
C  ****  We start by initialising PENELOPE...
C
      NMAT=1
      INFO=3
      PMFILE(1)=MFNAME
      WRITE(6,*) '  '
      WRITE(6,*) 'Processing material data. Please, wait...'
      OPEN(26,FILE='tables-class-II.dat')
      CALL PEINIT(EMAX,NMAT,26,INFO,PMFILE)
      IF(IRETRN.NE.0) THEN
        WRITE(6,'(A)') REASON
        WRITE(6,'(A,I5)') 'IRETRN =',IRETRN
        STOP
      ENDIF
      CLOSE(26)
      CALL GTAB2(MFNAME)
      CALL EPTAB2(MFNAME)
C
      WRITE(6,*) '  '
      WRITE(6,*) 'The output files have been generated.'
C
      END

C  *********************************************************************
C                       SUBROUTINE GTAB2
C  *********************************************************************
      SUBROUTINE GTAB2(MFNAME)
C
C  This subroutine prints tables of interaction properties of photons.
C
      USE PENELOPE_mod
      USE PENERROR_mod
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER*4 (I-N)
      CHARACTER*20 MFNAME
C  ****  Energy grid and interpolation constants for the current energy.
      COMMON/CEGRID/EMIN,EL,EU,ET(NEGP),DLEMP(NEGP),DLEMP1,DLFC,
     1  XEL,XE,XEK,KE
C  ****  Composition data.
      COMMON/COMPOS/STF(MAXMAT,30),ZT(MAXMAT),AT(MAXMAT),RHO(MAXMAT),
     1  VMOL(MAXMAT),IZ(MAXMAT,30),NELEM(MAXMAT)
C  ****  Photon simulation tables.
      COMMON/CGIMFP/SGRA(MAXMAT,NEGP),SGCO(MAXMAT,NEGP),
     1  SGPH(MAXMAT,NEGP),SGPP(MAXMAT,NEGP),SGAUX(MAXMAT,NEGP)
C  ****  Rayleigh scattering.
      PARAMETER (NQ=250,NEX=1024)
      COMMON/CGRA01/FF(MAXMAT,NQ),ERA(NEX),XSRA(MAXMAT,NEX),
     1    IED(NEGP),IEU(NEGP),NE
C  ****  Photoelectric cross sections.
      PARAMETER (NTP=12000)
      COMMON/CGPH00/EPH(NTP),XPH(NTP,17),IPHF(99),IPHL(99),NPHS(99),NCUR
C  ****  Pair and triplet production.
      COMMON/CGPP01/TRIP(MAXMAT,NEGP)
C
      PARAMETER (NDIM=12000)
      DIMENSION ER(NDIM),XSR(NDIM),X1(NDIM),Y1(NDIM),X2(NDIM),Y2(NDIM)
C
      M=1
C
      OPEN(11,FILE='PHgceil.tab')
      WRITE(11,1000) MFNAME
 1000 FORMAT(1X,'# Material data file: ',A20)
      WRITE(11,1101)
 1101 FORMAT(1X,'# PENELOPE >>>  Photelectric cross-section ceiling',
     1  ' (ph_ceil).')
      WRITE(11,1102)
 1102 FORMAT(1X,'#',/1X,'#   Energy',7X,'ph_ceil',
     1  /1X,'#    (eV) ',7X,'(cm**2)',/1X,'#',26('-'))
      DO IE=1,NEGP-1
        WRITE(11,'(1X,1P,3E13.5)') ET(IE),SGPH(M,IE)/VMOL(M)
        WRITE(11,'(1X,1P,3E13.5)') ET(IE+1),SGPH(M,IE)/VMOL(M)
      ENDDO
      CLOSE(11)
C
      OPEN(11,FILE='RAgceil.tab')
      WRITE(11,1000) MFNAME
      WRITE(11,1501)
 1501 FORMAT(1X,'# PENELOPE >>>  Rayleigh cross-section ceiling',
     1  ' (Ra_ceil).')
      WRITE(11,1502)
 1502 FORMAT(1X,'#',/1X,'#   Energy',7X,'Ra_ceil',
     1  /1X,'#    (eV) ',7X,'(cm**2)',/1X,'#',26('-'))
      DO IE=1,NEGP-1
        WRITE(11,'(1X,1P,3E13.5)') ET(IE),SGRA(M,IE)/VMOL(M)
        WRITE(11,'(1X,1P,3E13.5)') ET(IE+1),SGRA(M,IE)/VMOL(M)
      ENDDO
      CLOSE(11)
C
      OPEN(11,FILE='MACgRA.tab')
      WRITE(11,1000) MFNAME
      WRITE(11,1601)
 1601 FORMAT(1X,'# PENELOPE >>>  Rayleigh scattering x-section, ',
     1  'mac and mfp.',/1X,'#')
      WRITE(11,1602)
 1602 FORMAT(1X,'#   Energy',8X,'xs_Ra',7X,'mac_Ra',7X,'mfp_Ra',
     1  /1X,'#    (eV)',8X,'(cm**2)',5X,'(cm**2/g)',4X,'(g/cm**2)',
     1  /1X,'#',52('-'))
      DO IE=1,NE
        IF(ERA(IE).GT.DLEMP1) THEN
          VAL=EXP(XSRA(M,IE))
        WRITE(11,'(1X,1P,4E13.5)') EXP(ERA(IE)),VAL/VMOL(M),VAL/RHO(M),
     1    RHO(M)/VAL
        ENDIF
      ENDDO
      CLOSE(11)
C
      OPEN(12,FILE='TCSg.tab')
      WRITE(12,1000) MFNAME
      WRITE(12,1201)
 1201 FORMAT(1X,'# PENELOPE >>>  Photon cross sections (xs).')
      WRITE(12,1202)
 1202 FORMAT(1X,'#',/1X,'#   Energy',8X,'xs_Ra',8X,'xs_Co',8X,'xs_ph',
     1  8X,'xs_pp',7X,'xs_trip',6X,'xs_tot',/1X,'#    (eV) ',7X,
     1  '(cm**2)',6X,'(cm**2)',6X,'(cm**2)',6X,'(cm**2)',6X,'(cm**2)',
     1  6X,'(cm**2)',/1X,'#',91('-'))
C
      OPEN(14,FILE='MFPg.tab')
      WRITE(14,1000) MFNAME
      WRITE(14,1401)
 1401 FORMAT(1X,'# PENELOPE >>>  Photon mean free paths (mfp).')
      WRITE(14,1402)
 1402 FORMAT(1X,'#',/1X,'#   Energy',7X,'mfp_Ra',7X,'mfp_Co',7X,
     1   'mfp_ph',7X,'mfp_pp',7X,'mfp_tot',/1X,'#    (eV) ',6X,
     2  '(g/cm**2)',4X,'(g/cm**2)',4X,'(g/cm**2)',4X,'(g/cm**2)',4X,
     3  '(g/cm**2)',/1X,'#',78('-'))
C
      IZZ=IZ(M,1)
      IST=IPHF(IZZ)
      LST=IPHL(IZZ)
      NPHD=0
      DO I=IST,LST
        IF(EPH(I).GT.DLEMP1) THEN
          NPHD=NPHD+1
          ER(NPHD)=EXP(EPH(I))
          XSR(NPHD)=STF(M,1)*EXP(XPH(I,1))
        ENDIF
      ENDDO
      IF(NELEM(M).GT.1) THEN
        DO IEL=2,NELEM(M)
          N1=NPHD
          DO I=1,N1
            X1(I)=ER(I)
            Y1(I)=XSR(I)
          ENDDO
          IZZ=IZ(M,IEL)
          IST=IPHF(IZZ)
          LST=IPHL(IZZ)
          N2=0
          DO I=IST,LST
            N2=N2+1
            X2(N2)=EXP(EPH(I))
            Y2(N2)=STF(M,IEL)*EXP(XPH(I,1))
          ENDDO
          CALL MERGE2(X1,Y1,X2,Y2,ER,XSR,N1,N2,NPHD)
          IF(IRETRN.NE.0) THEN
            WRITE(6,'(A)') REASON
            WRITE(6,'(A,I5)') 'IRETRN =',IRETRN
            STOP
          ENDIF
        ENDDO
      ENDIF
      DO I=1,NPHD
        E=ER(I)
        IF(ABS(ER(I)-ER(I+1)).LT.1.0D-6*ER(I)) THEN
          E=E-1.0D-6
        ELSE IF(I.GT.1) THEN
          IF(ABS(ER(I)-ER(I-1)).LT.1.0D-6*ER(I)) E=E+1.0D-6
        ENDIF
        XEL=LOG(E)
        XE=1.0D0+(XEL-DLEMP1)*DLFC
        KE=XE
        XEK=XE-KE
C  ****  Binary search.
        II=IED(KE)
        IU=IEU(KE)
    1   IT=(II+IU)/2
        IF(XEL.GT.ERA(IT)) THEN
          II=IT
        ELSE
          IU=IT
        ENDIF
        IF(IU-II.GT.1) GO TO 1
        XTRA=EXP(XSRA(M,II)+(XSRA(M,II+1)-XSRA(M,II))*(XEL-ERA(II))
     1      /(ERA(II+1)-ERA(II)))/VMOL(M)
        RFPRA=XTRA*VMOL(M)/RHO(M)
        FPRA=1.0D0/RFPRA
        IF(KE.GT.0.AND.KE.LT.NEGP) THEN
          XSCO=EXP(SGCO(M,KE)+(SGCO(M,KE+1)-SGCO(M,KE))*XEK)/VMOL(M)
        ELSE
          CALL GCOaT(E,XSCO,M)
        ENDIF
        XSCO=MAX(XSCO,1.0D-35)
        RFPCO=XSCO*VMOL(M)/RHO(M)
        FPCO=1.0D0/RFPCO
        RFPPH=XSR(I)*VMOL(M)/RHO(M)
        FPPH=1.0D0/RFPPH
        IF(E.LT.1.023D6) THEN
          TXSPP=1.0D-35
          XSPPT=1.0D-35
          XSPP=1.0D-35
          RFPPP=1.0D-35
          FPPP=1.0D35
        ELSE
          TXSPP=EXP(SGPP(M,KE)+(SGPP(M,KE+1)-SGPP(M,KE))*XEK)/VMOL(M)
          PTRIP=TRIP(M,KE)+(TRIP(M,KE+1)-TRIP(M,KE))*XEK
          IF(PTRIP.GT.1.0D-10) THEN
            XSPPT=TXSPP*PTRIP
            XSPP=TXSPP*(1.0D0-PTRIP)
          ELSE
            XSPPT=1.0D-35
            XSPP=TXSPP
          ENDIF
          RFPPP=TXSPP*VMOL(M)/RHO(M)
          FPPP=1.0D0/RFPPP
        ENDIF
        XST=XTRA+XSCO+XSR(I)+TXSPP
        RFPT=XST*VMOL(M)/RHO(M)
        FPT=1.0D0/RFPT
        WRITE(12,'(1X,1P,7E13.5)') E,XTRA,XSCO,XSR(I),XSPP,XSPPT,XST
        WRITE(14,'(1X,1P,6E13.5)') E,FPRA,FPCO,FPPH,FPPP,FPT
      ENDDO
      CLOSE(12)
      CLOSE(14)
C
      RETURN
      END

C  *********************************************************************
C                       SUBROUTINE EPTAB2
C  *********************************************************************
      SUBROUTINE EPTAB2(MFNAME)
C
C  This subroutine prints tables of interaction properties of electrons
C  and positrons.
C
      USE PENELOPE_mod
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER*4 (I-N)
      CHARACTER*20 MFNAME
C  ****  Energy grid and interpolation constants for the current energy.
      COMMON/CEGRID/EMIN,EL,EU,ET(NEGP),DLEMP(NEGP),DLEMP1,DLFC,
     1  XEL,XE,XEK,KE
C  ****  Composition data.
      COMMON/COMPOS/STF(MAXMAT,30),ZT(MAXMAT),AT(MAXMAT),RHO(MAXMAT),
     1  VMOL(MAXMAT),IZ(MAXMAT,30),NELEM(MAXMAT)
C
C  ****  Electron simulation tables.
      COMMON/CEIMFP/SEHEL(MAXMAT,NEGP),SEHIN(MAXMAT,NEGP),
     1  SEISI(MAXMAT,NEGP),SEHBR(MAXMAT,NEGP),SEAUX(MAXMAT,NEGP),
     2  SETOT(MAXMAT,NEGP),CSTPE(MAXMAT,NEGP),RSTPE(MAXMAT,NEGP),
     3  DEL(MAXMAT,NEGP),W1E(MAXMAT,NEGP),W2E(MAXMAT,NEGP),
     4  DW1EL(MAXMAT,NEGP),DW2EL(MAXMAT,NEGP),
     5  RNDCE(MAXMAT,NEGP),AE(MAXMAT,NEGP),BE(MAXMAT,NEGP),
     6  T1E(MAXMAT,NEGP),T2E(MAXMAT,NEGP)
      COMMON/CLAS1E/TSTPE(MAXMAT,NEGP),TSTRE(MAXMAT,NEGP),
     1  TRL1E(MAXMAT,NEGP),TRL2E(MAXMAT,NEGP)
C  ****  Positron simulation tables.
      COMMON/CPIMFP/SPHEL(MAXMAT,NEGP),SPHIN(MAXMAT,NEGP),
     1  SPISI(MAXMAT,NEGP),SPHBR(MAXMAT,NEGP),SPAN(MAXMAT,NEGP),
     2  SPAUX(MAXMAT,NEGP),SPTOT(MAXMAT,NEGP),CSTPP(MAXMAT,NEGP),
     3  RSTPP(MAXMAT,NEGP),W1P(MAXMAT,NEGP),W2P(MAXMAT,NEGP),
     4  DW1PL(MAXMAT,NEGP),DW2PL(MAXMAT,NEGP),
     5  RNDCP(MAXMAT,NEGP),AP(MAXMAT,NEGP),BP(MAXMAT,NEGP),
     6  T1P(MAXMAT,NEGP),T2P(MAXMAT,NEGP)
      COMMON/CLAS1P/TSTPP(MAXMAT,NEGP),TSTRP(MAXMAT,NEGP),
     1  TRL1P(MAXMAT,NEGP),TRL2P(MAXMAT,NEGP)
C  ****  Total and transport cross sections.
      COMMON/CEEL00/EIT(NEGP),XE0(NEGP),XE1(NEGP),XE2(NEGP),XP0(NEGP),
     1  XP1(NEGP),XP2(NEGP),T1E0(NEGP),T2E0(NEGP),T1P0(NEGP),T2P0(NEGP),
     2  EITL(NEGP),FL(NEGP),A(NEGP),B(NEGP),C(NEGP),D(NEGP)
C
      PARAMETER (NEGRT=30)
      DIMENSION EGRD(NEGRT),ETS(300)
      DATA EGRD/ 1.00D0, 1.12D0, 1.25D0, 1.37D0, 1.50D0, 1.62D0, 1.75D0,
     1   1.87D0, 2.00D0, 2.25D0, 2.50D0, 2.75D0, 3.00D0, 3.25D0, 3.50D0,
     2   3.75D0, 4.00D0, 4.25D0, 4.50D0, 4.75D0, 5.00D0, 5.50D0, 6.00D0,
     3   6.50D0, 7.00D0, 7.50D0, 8.00D0, 8.50D0, 9.00D0, 1.00D1/
C
      M=1
C
C  ****  Energy mesh points (in eV).
C
      NIE=0
      IGRID=20
      FGRID=10.0D0
   10 IGRID=IGRID+1
      EV=EGRD(IGRID)*FGRID
      IF(IGRID.EQ.NEGRT) THEN
        IGRID=1
        FGRID=10.0D0*FGRID
      ENDIF
      IF(EV.GT.EL) THEN
        NIE=NIE+1
        ETS(NIE)=EV
      ENDIF
      IF(EV.LT.0.9999D9) GO TO 10
C
C  ****  Electrons.
C
      OPEN(12,FILE='HMFPe.tab')
      WRITE(12,1000) MFNAME
 1000 FORMAT(1X,'# Material data file: ',A20,/1X,'# ')
      WRITE(12,1201)
 1201 FORMAT(1X,'# PENELOPE >>>  Electron mean free paths (mfp) for',
     1  'hard interactions'/1X,'# ')
      WRITE(12,1202)
 1202 FORMAT(1X,'# NOTE: The mfp for inner shell ionization (isi) is',
     2  ' listed only for',/1X,'#',7X,'completeness. The mfp for inela',
     3  'stic collisions (in) has been',/1X,'#',7X,'calculated by',
     4  ' considering all inelastic events, including isi.')
      WRITE(12,1203)
 1203 FORMAT(1X,'#',/1X,'#   Energy',7X,'mfp_el',7X,'mfp_in',7X,
     1  'mfp_br',7X,'mfp_isi',6X,'mfp_tot',/1X,'#    (eV) ',6X,
     2  '(g/cm**2)',4X,'(g/cm**2)',4X,'(g/cm**2)',4X,'(g/cm**2)',4X,
     3  '(g/cm**2)',/1X,'#',78('-'))
C
      OPEN(13,FILE='HCOLe.tab')
      WRITE(13,1000) MFNAME
      WRITE(13,1301)
 1301 FORMAT(1X,'# PENELOPE >>>  Average numbers <N> of hard interac',
     1 'tions of electrons'/1X,'# ')
      WRITE(13,1302)
 1302 FORMAT(1X,'# NOTE: The <N> of inner shell ionizations (isi) is',
     2  ' listed only for',/1X,'#',7X,'completeness. The <N> of inela',
     3  'stic collisions (in) has been',/1X,'#',7X,'calculated by',
     4  ' considering all inelastic events, including isi.')
      WRITE(13,1303)
 1303 FORMAT(1X,'#',/1X,'#   E (eV)',7X,'<N>_el',7X,'<N>_in',7X,
     1  '<N>_br',7X,'<N>_isi',6X,'<N>_tot',/1X,'#',78('-'))
C
      DO IES=1,NIE
        E=ETS(IES)
        IF(E.LT.EL) THEN
          EE=EL
        ELSE IF (E.GT.EU) THEN
          EE=EU
        ELSE
          EE=E
        ENDIF
        XEL=LOG(EE)
        XE=1.0D0+(XEL-DLEMP1)*DLFC
        KE=XE
        IF(KE.LT.1) KE=1
        IF(KE.GE.NEGP) KE=NEGP-1
        XEK=XE-KE
C
        XSEL=EXP(SEHEL(M,KE)+(SEHEL(M,KE+1)-SEHEL(M,KE))*XEK)/VMOL(M)
        XSIN=EXP(SEHIN(M,KE)+(SEHIN(M,KE+1)-SEHIN(M,KE))*XEK)/VMOL(M)
        XSBR=EXP(SEHBR(M,KE)+(SEHBR(M,KE+1)-SEHBR(M,KE))*XEK)/VMOL(M)
        XSSI=EXP(SEISI(M,KE)+(SEISI(M,KE+1)-SEISI(M,KE))*XEK)/VMOL(M)
        XSTOT=XSEL+XSIN+XSBR+XSSI
        FPEL=RHO(M)/(XSEL*VMOL(M))
        FPIN=RHO(M)/((XSIN+XSSI)*VMOL(M))
        FPBR=RHO(M)/(XSBR*VMOL(M))
        FPSI=RHO(M)/(XSSI*VMOL(M))
        FPTOT=RHO(M)/(XSTOT*VMOL(M))
        WRITE(12,'(1X,1P,7E13.5)') E,FPEL,FPIN,FPBR,FPSI,FPTOT
C
        A2=AVNCOL(E,1,M,2)
        A3=AVNCOL(E,1,M,3)
        A4=AVNCOL(E,1,M,4)
        A5=AVNCOL(E,1,M,5)
        ATOT=AVNCOL(E,1,M,0)
        WRITE(13,'(1X,1P,6E13.5)') E,A2,A3,A4,A5,ATOT
        IF(ATOT.GT.1.0D0) THEN
          ASUM=MAX(A2+A3+A4,1.0D-35)
          IF(ABS(ASUM/ATOT-1.0D0).GT.0.1D0) THEN
            WRITE(6,'(A)') 'Possible error in the interpolation of MFPs'
            WRITE(6,'(A,1P,E13.5)') '  Electron, E/eV =',E
            WRITE(6,'(A,1P,E13.5)') '  <N>_tot from partial MFPs =',ASUM
            WRITE(6,'(A,1P,E13.5)') '  <N>_tot from total MFPs   =',ATOT
          ENDIF
        ENDIF
      ENDDO
      CLOSE(12)
      CLOSE(13)
C
C  ****  Positrons.
C
      OPEN(22,FILE='HMFPp.tab')
      WRITE(22,1000) MFNAME
      WRITE(22,2201)
 2201 FORMAT(1X,'# PENELOPE >>>  Positron mean free paths (mfp) for',
     1  'hard interactions'/1X,'# ')
      WRITE(22,1202)
      WRITE(22,2203)
 2203 FORMAT(1X,'#',/1X,'#   Energy',7X,'mfp_el',7X,'mfp_in',7X,
     1  'mfp_br',7X,'mfp_an',7X,'mfp_isi',6X,'mfp_tot',/1X,'#    (eV) ',
     2  6X,'(g/cm**2)',4X,'(g/cm**2)',4X,'(g/cm**2)',4X,'(g/cm**2)',4X,
     3  '(g/cm**2)',4X,'(g/cm**2)',/1X,'#',91('-'))
C
      OPEN(23,FILE='HCOLp.tab')
      WRITE(23,1000) MFNAME
      WRITE(23,2301)
 2301 FORMAT(1X,'# PENELOPE >>>  Average numbers <N> of hard interac',
     1 'tions of positrons'/1X,'# ')
      WRITE(23,1302)
      WRITE(23,2303)
 2303 FORMAT(1X,'#',/1X,'#   E (eV)',7X,'<N>_el',7X,'<N>_in',7X,
     1  '<N>_br',7X,'<N>_isi',6X,'<N>_an',7X,'<N>_tot',/1X,'#',91('-'))
C
      DO IES=1,NIE
        E=ETS(IES)
        IF(E.LT.EL) THEN
          EE=EL
        ELSE IF (E.GT.EU) THEN
          EE=EU
        ELSE
          EE=E
        ENDIF
        XEL=LOG(EE)
        XE=1.0D0+(XEL-DLEMP1)*DLFC
        KE=XE
        IF(KE.LT.1) KE=1
        IF(KE.GE.NEGP) KE=NEGP-1
        XEK=XE-KE
C
        XSEL=EXP(SPHEL(M,KE)+(SPHEL(M,KE+1)-SPHEL(M,KE))*XEK)/VMOL(M)
        XSIN=EXP(SPHIN(M,KE)+(SPHIN(M,KE+1)-SPHIN(M,KE))*XEK)/VMOL(M)
        XSBR=EXP(SPHBR(M,KE)+(SPHBR(M,KE+1)-SPHBR(M,KE))*XEK)/VMOL(M)
        XSSI=EXP(SPISI(M,KE)+(SPISI(M,KE+1)-SPISI(M,KE))*XEK)/VMOL(M)
        XSAN=EXP(SPAN(M,KE)+(SPAN(M,KE+1)-SPAN(M,KE))*XEK)/VMOL(M)
        XSTOT=XSEL+XSIN+XSBR+XSAN+XSSI
        FPEL=RHO(M)/(XSEL*VMOL(M))
        FPIN=RHO(M)/((XSIN+XSSI)*VMOL(M))
        FPBR=RHO(M)/(XSBR*VMOL(M))
        FPAN=RHO(M)/(XSAN*VMOL(M))
        FPSI=RHO(M)/(XSSI*VMOL(M))
        FPTOT=RHO(M)/(XSTOT*VMOL(M))
        WRITE(22,'(1X,1P,7E13.5)') E,FPEL,FPIN,FPBR,FPAN,FPSI,FPTOT
C
        A2=AVNCOL(E,3,M,2)
        A3=AVNCOL(E,3,M,3)
        A4=AVNCOL(E,3,M,4)
        A5=AVNCOL(E,3,M,5)
        A6=AVNCOL(E,3,M,6)
        ATOT=AVNCOL(E,3,M,0)
        WRITE(23,'(1X,1P,7E13.5)') E,A2,A3,A4,A5,A6,ATOT
        IF(ATOT.GT.1.0D0) THEN
          ASUM=MAX(A2+A3+A4+A6,1.0D-35)
          IF(ABS(ASUM/ATOT-1.0D0).GT.0.1D0) THEN
            WRITE(6,'(A)') 'Possible error in the interpolation of MFPs'
            WRITE(6,'(A,1P,E13.5)') '  Positron, E/eV =',E
            WRITE(6,'(A,1P,E13.5)') '  <N>_tot from partial MFPs =',ASUM
            WRITE(6,'(A,1P,E13.5)') '  <N>_tot from total MFPs   =',ATOT
          ENDIF
        ENDIF
      ENDDO
      CLOSE(22)
      CLOSE(23)
C
      RETURN
      END
